@@ -1,26 +1,39 @@
 use crate::{
     input::user::KeyBinding,
+    markdown::elements::SnippetLanguage,
     media::{emulator::TerminalEmulator, kitty::KittyMode},
     GraphicsMode,
 };
 use clap::ValueEnum;
+use schemars::JsonSchema;
 use serde::Deserialize;
-use std::{fs, io, path::Path};
+use std::{
+    collections::{BTreeMap, HashMap},
+    fs, io,
+    path::Path,
+};
 
-#[derive(Clone, Debug, Default, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
 #[serde(deny_unknown_fields)]
 pub struct Config {
     #[serde(default)]
+    #[doc = "The default configuration for the presentation."]
     pub defaults: DefaultsConfig,
 
     #[serde(default)]
     pub typst: TypstConfig,
 
+    #[serde(default)]
+    pub mermaid: MermaidConfig,
+
     #[serde(default)]
     pub options: OptionsConfig,
 
     #[serde(default)]
     pub bindings: KeyBindingsConfig,
+
+    #[serde(default)]
+    pub snippet: SnippetConfig,
 }
 
 impl Config {
@@ -45,17 +58,22 @@ pub enum ConfigLoadError {
     Invalid(#[from] serde_yaml::Error),
 }
 
-#[derive(Clone, Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize, JsonSchema)]
 #[serde(deny_unknown_fields)]
 pub struct DefaultsConfig {
+    /// The theme to use by default in every presentation unless overridden.
     pub theme: Option<String>,
 
+    /// Override the terminal font size when in windows or when using sixel.
     #[serde(default = "default_font_size")]
+    #[validate(range(min = 1))]
     pub terminal_font_size: u8,
 
+    /// The image protocol to use.
     #[serde(default)]
     pub image_protocol: ImageProtocol,
 
+    /// Validate that the presentation does not overflow the terminal screen.
     #[serde(default)]
     pub validate_overflows: ValidateOverflows,
 }
@@ -75,7 +93,7 @@ fn default_font_size() -> u8 {
     16
 }
 
-#[derive(Clone, Debug, Default, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
 #[serde(rename_all = "snake_case")]
 pub enum ValidateOverflows {
     #[default]
@@ -85,7 +103,7 @@ pub enum ValidateOverflows {
     WhenDeveloping,
 }
 
-#[derive(Clone, Debug, Default, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
 #[serde(deny_unknown_fields)]
 pub struct OptionsConfig {
     /// Whether slides are automatically terminated when a slide title is found.
@@ -94,6 +112,9 @@ pub struct OptionsConfig {
     /// The prefix to use for commands.
     pub command_prefix: Option<String>,
 
+    /// The prefix to use for image attributes.
+    pub image_attributes_prefix: Option<String>,
+
     /// Show all lists incrementally, by implicitly adding pauses in between elements.
     pub incremental_lists: Option<bool>,
 
@@ -104,9 +125,51 @@ pub struct OptionsConfig {
     pub strict_front_matter_parsing: Option<bool>,
 }
 
-#[derive(Clone, Debug, Deserialize)]
+#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct SnippetConfig {
+    /// The properties for snippet execution.
+    #[serde(default)]
+    pub exec: SnippetExecConfig,
+
+    /// The properties for snippet auto rendering.
+    #[serde(default)]
+    pub render: SnippetRenderConfig,
+}
+
+#[derive(Clone, Debug, Default, Deserialize, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct SnippetExecConfig {
+    /// Whether to enable snippet execution.
+    pub enable: bool,
+
+    /// Custom snippet executors.
+    #[serde(default)]
+    pub custom: BTreeMap<SnippetLanguage, LanguageSnippetExecutionConfig>,
+}
+
+#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct SnippetRenderConfig {
+    /// The number of threads to use when rendering.
+    #[serde(default = "default_snippet_render_threads")]
+    pub threads: usize,
+}
+
+impl Default for SnippetRenderConfig {
+    fn default() -> Self {
+        Self { threads: default_snippet_render_threads() }
+    }
+}
+
+pub(crate) fn default_snippet_render_threads() -> usize {
+    2
+}
+
+#[derive(Clone, Debug, Deserialize, JsonSchema)]
 #[serde(deny_unknown_fields)]
 pub struct TypstConfig {
+    /// The pixels per inch when rendering latex/typst formulas.
     #[serde(default = "default_typst_ppi")]
     pub ppi: u32,
 }
@@ -117,19 +180,65 @@ impl Default for TypstConfig {
     }
 }
 
-fn default_typst_ppi() -> u32 {
+pub(crate) fn default_typst_ppi() -> u32 {
     300
 }
 
-#[derive(Clone, Debug, Default, Deserialize, ValueEnum)]
+#[derive(Clone, Debug, Deserialize, JsonSchema)]
+#[serde(deny_unknown_fields)]
+pub struct MermaidConfig {
+    /// The scaling parameter to be used in the mermaid CLI.
+    #[serde(default = "default_mermaid_scale")]
+    pub scale: u32,
+}
+
+impl Default for MermaidConfig {
+    fn default() -> Self {
+        Self { scale: default_mermaid_scale() }
+    }
+}
+
+pub(crate) fn default_mermaid_scale() -> u32 {
+    2
+}
+
+/// The snippet execution configuration for a specific programming language.
+#[derive(Clone, Debug, Deserialize, JsonSchema)]
+pub struct LanguageSnippetExecutionConfig {
+    /// The filename to use for the snippet input file.
+    pub filename: String,
+
+    /// The environment variables to set before invoking every command.
+    #[serde(default)]
+    pub environment: HashMap<String, String>,
+
+    /// The commands to be run when executing snippets for this programming language.
+    pub commands: Vec<Vec<String>>,
+}
+
+#[derive(Clone, Debug, Default, Deserialize, ValueEnum, JsonSchema)]
 #[serde(rename_all = "kebab-case")]
 pub enum ImageProtocol {
+    /// Automatically detect the best image protocol to use.
     #[default]
     Auto,
+
+    /// Use the iTerm2 image protocol.
     Iterm2,
+
+    /// Use the kitty protocol in "local" mode, meaning both presenterm and the terminal run in the
+    /// same host and can share the filesystem to communicate.
     KittyLocal,
+
+    /// Use the kitty protocol in "remote" mode, meaning presenterm and the terminal run in
+    /// different hosts and therefore can only communicate via terminal escape codes.
     KittyRemote,
+
+    /// Use the sixel protocol. Note that this requires compiling presenterm using the --features
+    /// sixel flag.
     Sixel,
+
+    /// The default image protocol to use when no other is specified.
     AsciiBlocks,
 }
 
@@ -161,39 +270,64 @@ impl TryFrom<&ImageProtocol> for GraphicsMode {
     }
 }
 
-#[derive(Clone, Debug, Deserialize)]
+#[derive(Clone, Debug, Deserialize, JsonSchema)]
 #[serde(deny_unknown_fields)]
 pub struct KeyBindingsConfig {
+    /// The keys that cause the presentation to move forwards.
     #[serde(default = "default_next_bindings")]
     pub(crate) next: Vec<KeyBinding>,
 
+    /// The keys that cause the presentation to jump to the next slide "fast".
+    ///
+    /// "fast" means for slides that contain pauses, we will only jump between the first and last
+    /// pause rather than going through each individual one.
+    #[serde(default = "default_next_fast_bindings")]
+    pub(crate) next_fast: Vec<KeyBinding>,
+
+    /// The keys that cause the presentation to move backwards.
     #[serde(default = "default_previous_bindings")]
     pub(crate) previous: Vec<KeyBinding>,
 
+    /// The keys that cause the presentation to move backwards "fast".
+    ///
+    /// "fast" means for slides that contain pauses, we will only jump between the first and last
+    /// pause rather than going through each individual one.
+    #[serde(default = "default_previous_fast_bindings")]
+    pub(crate) previous_fast: Vec<KeyBinding>,
+
+    /// The key binding to jump to the first slide.
     #[serde(default = "default_first_slide_bindings")]
     pub(crate) first_slide: Vec<KeyBinding>,
 
+    /// The key binding to jump to the last slide.
     #[serde(default = "default_last_slide_bindings")]
     pub(crate) last_slide: Vec<KeyBinding>,
 
+    /// The key binding to jump to a specific slide.
     #[serde(default = "default_go_to_slide_bindings")]
     pub(crate) go_to_slide: Vec<KeyBinding>,
 
+    /// The key binding to execute a piece of shell code.
     #[serde(default = "default_execute_code_bindings")]
     pub(crate) execute_code: Vec<KeyBinding>,
 
+    /// The key binding to reload the presentation.
     #[serde(default = "default_reload_bindings")]
     pub(crate) reload: Vec<KeyBinding>,
 
+    /// The key binding to toggle the slide index modal.
     #[serde(default = "default_toggle_index_bindings")]
     pub(crate) toggle_slide_index: Vec<KeyBinding>,
 
+    /// The key binding to toggle the key bindings modal.
     #[serde(default = "default_toggle_bindings_modal_bindings")]
     pub(crate) toggle_bindings: Vec<KeyBinding>,
 
+    /// The key binding to close the currently open modal.
     #[serde(default = "default_close_modal_bindings")]
     pub(crate) close_modal: Vec<KeyBinding>,
 
+    /// The key binding to close the application.
     #[serde(default = "default_exit_bindings")]
     pub(crate) exit: Vec<KeyBinding>,
 }
@@ -202,7 +336,9 @@ impl Default for KeyBindingsConfig {
     fn default() -> Self {
         Self {
             next: default_next_bindings(),
+            next_fast: default_next_fast_bindings(),
             previous: default_previous_bindings(),
+            previous_fast: default_previous_fast_bindings(),
             first_slide: default_first_slide_bindings(),
             last_slide: default_last_slide_bindings(),
             go_to_slide: default_go_to_slide_bindings(),
@@ -228,10 +364,18 @@ fn default_next_bindings() -> Vec<KeyBinding> {
     make_keybindings(["l", "j", "<right>", "<page_down>", "<down>", " "])
 }
 
+fn default_next_fast_bindings() -> Vec<KeyBinding> {
+    make_keybindings(["n"])
+}
+
 fn default_previous_bindings() -> Vec<KeyBinding> {
     make_keybindings(["h", "k", "<left>", "<page_up>", "<up>"])
 }
 
+fn default_previous_fast_bindings() -> Vec<KeyBinding> {
+    make_keybindings(["p"])
+}
+
 fn default_first_slide_bindings() -> Vec<KeyBinding> {
     make_keybindings(["gg"])
 }
