@@ -1,4 +1,5 @@
 use crate::{
+    execute::SnippetExecutor,
     input::{
         source::Command,
         user::{CommandKeyBindings, UserInput},
@@ -7,9 +8,9 @@ use crate::{
     presentation::Presentation,
     processing::builder::{BuildError, PresentationBuilder},
     render::{draw::TerminalDrawer, terminal::TerminalWrite},
-    ImageRegistry, MarkdownParser, PresentationBuilderOptions, PresentationTheme, Resources, Themes, TypstRender,
+    ImageRegistry, MarkdownParser, PresentationBuilderOptions, PresentationTheme, Resources, Themes, ThirdPartyRender,
 };
-use std::io;
+use std::{io, rc::Rc};
 
 const PRESENTATION: &str = r#"
 # Header 1
@@ -99,13 +100,15 @@ impl<W: TerminalWrite> ThemesDemo<W> {
     ) -> Result<Presentation, BuildError> {
         let image_registry = ImageRegistry::default();
         let mut resources = Resources::new("non_existent", image_registry.clone());
-        let mut typst = TypstRender::default();
+        let mut third_party = ThirdPartyRender::default();
         let options = PresentationBuilderOptions::default();
+        let executer = Rc::new(SnippetExecutor::default());
         let bindings_config = Default::default();
         let builder = PresentationBuilder::new(
             theme,
             &mut resources,
-            &mut typst,
+            &mut third_party,
+            executer,
             &self.themes,
             image_registry,
             bindings_config,
