@@ -1,14 +1,16 @@
 use crate::{
     custom::KeyBindingsConfig,
+    execute::SnippetExecutor,
     markdown::parse::ParseError,
     media::{
         image::{Image, ImageSource},
         printer::{ImageResource, ResourceProperties},
     },
-    presentation::{Presentation, RenderOperation},
+    presentation::{AsRenderOperations, Presentation, RenderAsyncState, RenderOperation},
     processing::builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
+    render::properties::WindowSize,
+    third_party::ThirdPartyRender,
     tools::{ExecutionError, ThirdPartyTools},
-    typst::TypstRender,
     MarkdownParser, PresentationTheme, Resources,
 };
 use base64::{engine::general_purpose::STANDARD, Engine};
@@ -16,19 +18,23 @@ use image::{codecs::png::PngEncoder, DynamicImage, ImageEncoder, ImageError};
 use semver::Version;
 use serde::Serialize;
 use std::{
-    env, fs,
-    io::{self},
+    env, fs, io, iter,
     path::{Path, PathBuf},
+    rc::Rc,
+    thread::sleep,
+    time::Duration,
 };
 
 const MINIMUM_EXPORTER_VERSION: Version = Version::new(0, 2, 0);
+const ASYNC_RENDER_WAIT_COUNT: usize = 8;
 
 /// Allows exporting presentations into PDF.
 pub struct Exporter<'a> {
     parser: MarkdownParser<'a>,
     default_theme: &'a PresentationTheme,
     resources: Resources,
-    typst: TypstRender,
+    third_party: ThirdPartyRender,
+    code_executor: Rc<SnippetExecutor>,
     themes: Themes,
     options: PresentationBuilderOptions,
 }
@@ -39,11 +45,12 @@ impl<'a> Exporter<'a> {
         parser: MarkdownParser<'a>,
         default_theme: &'a PresentationTheme,
         resources: Resources,
-        typst: TypstRender,
+        third_party: ThirdPartyRender,
+        code_executor: Rc<SnippetExecutor>,
         themes: Themes,
         options: PresentationBuilderOptions,
     ) -> Self {
-        Self { parser, default_theme, resources, typst, themes, options }
+        Self { parser, default_theme, resources, third_party, code_executor, themes, options }
     }
 
     /// Export the given presentation into PDF.
@@ -52,7 +59,9 @@ impl<'a> Exporter<'a> {
     pub fn export_pdf(&mut self, presentation_path: &Path, extra_args: &[&str]) -> Result<(), ExportError> {
         Self::validate_exporter_version()?;
 
+        println!("Analyzing presentation...");
         let metadata = self.generate_metadata(presentation_path)?;
+        println!("Invoking presenterm-export...");
         Self::execute_exporter(metadata, extra_args)?;
         Ok(())
     }
@@ -82,7 +91,8 @@ impl<'a> Exporter<'a> {
         let mut presentation = PresentationBuilder::new(
             self.default_theme,
             &mut self.resources,
-            &mut self.typst,
+            &mut self.third_party,
+            self.code_executor.clone(),
             &self.themes,
             Default::default(),
             KeyBindingsConfig::default(),
@@ -90,9 +100,10 @@ impl<'a> Exporter<'a> {
         )
         .build(elements)?;
 
+        let async_renders = Self::count_async_render_operations(&presentation);
         let images = Self::build_image_metadata(&mut presentation)?;
         Self::validate_theme_colors(&presentation)?;
-        let commands = Self::build_capture_commands(presentation);
+        let commands = Self::build_capture_commands(presentation, async_renders);
         let metadata = ExportMetadata { commands, presentation_path: path, images };
         Ok(metadata)
     }
@@ -102,14 +113,14 @@ impl<'a> Exporter<'a> {
         let presenterm_path = presenterm_path.display().to_string();
         let presentation_path = metadata.presentation_path.display().to_string();
         let metadata = serde_json::to_vec(&metadata).expect("serialization failed");
-        let mut args = vec![&presenterm_path, "--export"];
+        let mut args = vec![&presenterm_path, "--enable-export-mode"];
         args.extend(extra_args);
         args.push(&presentation_path);
         ThirdPartyTools::presenterm_export(&args).stdin(metadata).run()?;
         Ok(())
     }
 
-    fn build_capture_commands(mut presentation: Presentation) -> Vec<CaptureCommand> {
+    fn build_capture_commands(mut presentation: Presentation, async_renders: usize) -> Vec<CaptureCommand> {
         let mut commands = Vec::new();
         let slide_chunks: Vec<_> = presentation.iter_slides().map(|slide| slide.iter_chunks().count()).collect();
         let mut next_slide = |commands: &mut Vec<CaptureCommand>| {
@@ -117,6 +128,7 @@ impl<'a> Exporter<'a> {
             commands.push(CaptureCommand::WaitForChange);
             presentation.jump_next();
         };
+        commands.extend(iter::repeat(CaptureCommand::WaitForChange).take(ASYNC_RENDER_WAIT_COUNT * async_renders));
         for chunks in slide_chunks {
             for _ in 0..chunks - 1 {
                 next_slide(&mut commands);
@@ -127,6 +139,15 @@ impl<'a> Exporter<'a> {
         commands
     }
 
+    fn count_async_render_operations(presentation: &Presentation) -> usize {
+        presentation
+            .iter_slides()
+            .map(|slide| {
+                slide.iter_visible_operations().filter(|op| matches!(op, RenderOperation::RenderAsync(_))).count()
+            })
+            .sum()
+    }
+
     fn build_image_metadata(presentation: &mut Presentation) -> Result<Vec<ImageMetadata>, ExportError> {
         let mut replacer = ImageReplacer::default();
         replacer.replace_presentation_images(presentation);
@@ -148,7 +169,7 @@ impl<'a> Exporter<'a> {
                         resource.as_bytes(),
                         dimensions.0,
                         dimensions.1,
-                        resource.color(),
+                        resource.color().into(),
                     )?;
                     let contents = Some(STANDARD.encode(buffer));
                     ImageMetadata { path: None, color: image.color, contents }
@@ -161,7 +182,7 @@ impl<'a> Exporter<'a> {
 
     fn validate_theme_colors(presentation: &Presentation) -> Result<(), ExportError> {
         for slide in presentation.iter_slides() {
-            for operation in slide.iter_operations() {
+            for operation in slide.iter_visible_operations() {
                 let RenderOperation::SetColors(colors) = operation else {
                     continue;
                 };
@@ -245,11 +266,39 @@ pub(crate) struct ImageReplacer {
 impl ImageReplacer {
     pub(crate) fn replace_presentation_images(&mut self, presentation: &mut Presentation) {
         let callback = |operation: &mut RenderOperation| {
-            let RenderOperation::RenderImage(image, properties) = operation else {
-                return;
+            match operation {
+                RenderOperation::RenderImage(image, properties) => {
+                    let replacement = self.replace_image(image.clone());
+                    *operation = RenderOperation::RenderImage(replacement, properties.clone());
+                }
+                RenderOperation::RenderAsync(inner) => {
+                    loop {
+                        match inner.poll_state() {
+                            RenderAsyncState::NotStarted => return,
+                            RenderAsyncState::Rendering { .. } => {
+                                sleep(Duration::from_millis(200));
+                                continue;
+                            }
+                            RenderAsyncState::Rendered | RenderAsyncState::JustFinishedRendering => break,
+                        };
+                    }
+
+                    let window_size = WindowSize { rows: 0, columns: 0, width: 0, height: 0 };
+                    let mut new_operations = Vec::new();
+                    for operation in inner.as_render_operations(&window_size) {
+                        if let RenderOperation::RenderImage(image, properties) = operation {
+                            let image = self.replace_image(image);
+                            new_operations.push(RenderOperation::RenderImage(image, properties));
+                        } else {
+                            new_operations.push(operation);
+                        }
+                    }
+                    // Replace this operation with a new operation that contains the replaced image
+                    // and any other unmodified operations.
+                    *operation = RenderOperation::RenderDynamic(Rc::new(RenderMany(new_operations)));
+                }
+                _ => (),
             };
-            let replacement = self.replace_image(image.clone());
-            *operation = RenderOperation::RenderImage(replacement, properties.clone());
         };
 
         presentation.mutate_operations(callback);
@@ -287,6 +336,19 @@ impl Default for ImageReplacer {
     }
 }
 
+#[derive(Debug)]
+struct RenderMany(Vec<RenderOperation>);
+
+impl AsRenderOperations for RenderMany {
+    fn as_render_operations(&self, _: &WindowSize) -> Vec<RenderOperation> {
+        self.0.clone()
+    }
+
+    fn diffable_content(&self) -> Option<&str> {
+        None
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -298,10 +360,11 @@ mod test {
         let parser = MarkdownParser::new(&arena);
         let theme = PresentationThemeSet::default().load_by_name("dark").unwrap();
         let resources = Resources::new("examples", Default::default());
-        let typst = TypstRender::default();
+        let third_party = ThirdPartyRender::default();
+        let code_executor = Default::default();
         let themes = Themes::default();
         let options = PresentationBuilderOptions { allow_mutations: false, ..Default::default() };
-        let mut exporter = Exporter::new(parser, &theme, resources, typst, themes, options);
+        let mut exporter = Exporter::new(parser, &theme, resources, third_party, code_executor, themes, options);
         exporter.extract_metadata(content, Path::new(path)).expect("metadata extraction failed")
     }
 
