@@ -1,6 +1,7 @@
 use super::source::{Command, CommandDiscriminants};
 use crate::custom::KeyBindingsConfig;
 use crossterm::event::{poll, read, Event, KeyCode, KeyEvent, KeyEventKind, KeyModifiers};
+use schemars::JsonSchema;
 use serde_with::DeserializeFromStr;
 use std::{fmt, io, iter, mem, str::FromStr, time::Duration};
 
@@ -74,7 +75,9 @@ impl CommandKeyBindings {
         let command = match discriminant {
             Redraw => Command::Redraw,
             Next => Command::Next,
+            NextFast => Command::NextFast,
             Previous => Command::Previous,
+            PreviousFast => Command::PreviousFast,
             FirstSlide => Command::FirstSlide,
             LastSlide => Command::LastSlide,
             GoToSlide => {
@@ -85,7 +88,7 @@ impl CommandKeyBindings {
                     MatchContext::Number(number) => Command::GoToSlide(number),
                 }
             }
-            RenderWidgets => Command::RenderWidgets,
+            RenderAsyncOperations => Command::RenderAsyncOperations,
             Exit => Command::Exit,
             Reload => Command::Reload,
             HardReload => Command::HardReload,
@@ -123,7 +126,9 @@ impl TryFrom<KeyBindingsConfig> for CommandKeyBindings {
         }
         let bindings: Vec<_> = iter::empty()
             .chain(zip(CommandDiscriminants::Next, config.next))
+            .chain(zip(CommandDiscriminants::NextFast, config.next_fast))
             .chain(zip(CommandDiscriminants::Previous, config.previous))
+            .chain(zip(CommandDiscriminants::PreviousFast, config.previous_fast))
             .chain(zip(CommandDiscriminants::FirstSlide, config.first_slide))
             .chain(zip(CommandDiscriminants::LastSlide, config.last_slide))
             .chain(zip(CommandDiscriminants::GoToSlide, config.go_to_slide))
@@ -131,7 +136,7 @@ impl TryFrom<KeyBindingsConfig> for CommandKeyBindings {
             .chain(zip(CommandDiscriminants::HardReload, config.reload))
             .chain(zip(CommandDiscriminants::ToggleSlideIndex, config.toggle_slide_index))
             .chain(zip(CommandDiscriminants::ToggleKeyBindingsConfig, config.toggle_bindings))
-            .chain(zip(CommandDiscriminants::RenderWidgets, config.execute_code))
+            .chain(zip(CommandDiscriminants::RenderAsyncOperations, config.execute_code))
             .chain(zip(CommandDiscriminants::CloseModal, config.close_modal))
             .collect();
         Self::validate_conflicts(bindings.iter().map(|binding| &binding.0))?;
@@ -155,8 +160,8 @@ enum BindingMatch {
     None,
 }
 
-#[derive(Clone, Debug, PartialEq, Eq, DeserializeFromStr)]
-pub struct KeyBinding(Vec<KeyMatcher>);
+#[derive(Clone, Debug, PartialEq, Eq, DeserializeFromStr, JsonSchema)]
+pub struct KeyBinding(#[schemars(with = "String")] Vec<KeyMatcher>);
 
 impl KeyBinding {
     fn match_events(&self, mut events: &[KeyEvent]) -> BindingMatch {
@@ -532,7 +537,7 @@ mod test {
     )]
     fn match_number(#[case] pattern: &str, #[case] events: &[KeyEvent]) {
         let binding = KeyBinding::from_str(pattern).expect("failed to parse");
-        let result = binding.match_events(&events);
+        let result = binding.match_events(events);
         let BindingMatch::Full(MatchContext::Number(number)) = result else {
             panic!("unexpected match: {result:?}");
         };
