@@ -17,12 +17,13 @@ pub(crate) mod render;
 pub(crate) mod resource;
 pub(crate) mod style;
 pub(crate) mod theme;
+pub(crate) mod third_party;
 pub(crate) mod tools;
-pub(crate) mod typst;
 
 pub use crate::{
     custom::{Config, ImageProtocol, ValidateOverflows},
     demo::ThemesDemo,
+    execute::SnippetExecutor,
     export::{ExportError, Exporter},
     input::source::CommandSource,
     markdown::parse::MarkdownParser,
@@ -32,5 +33,5 @@ pub use crate::{
     render::highlighting::{CodeHighlighter, HighlightThemeSet},
     resource::Resources,
     theme::{LoadThemeError, PresentationTheme, PresentationThemeSet},
-    typst::TypstRender,
+    third_party::{ThirdPartyConfigs, ThirdPartyRender},
 };
