@@ -3,13 +3,15 @@ use comrak::Arena;
 use directories::ProjectDirs;
 use presenterm::{
     CommandSource, Config, Exporter, GraphicsMode, HighlightThemeSet, ImagePrinter, ImageProtocol, ImageRegistry,
-    LoadThemeError, MarkdownParser, PresentMode, PresentationBuilderOptions, PresentationTheme, PresentationThemeSet,
-    Presenter, PresenterOptions, Resources, Themes, ThemesDemo, TypstRender, ValidateOverflows,
+    MarkdownParser, PresentMode, PresentationBuilderOptions, PresentationTheme, PresentationThemeSet, Presenter,
+    PresenterOptions, Resources, SnippetExecutor, Themes, ThemesDemo, ThirdPartyConfigs, ThirdPartyRender,
+    ValidateOverflows,
 };
 use std::{
     env, io,
     path::{Path, PathBuf},
     rc::Rc,
+    sync::Arc,
 };
 
 const DEFAULT_THEME: &str = "dark";
@@ -31,9 +33,13 @@ struct Cli {
     #[clap(long, hide = true)]
     generate_pdf_metadata: bool,
 
+    /// Generate a JSON schema for the configuration file.
+    #[clap(long)]
+    generate_config_file_schema: bool,
+
     /// Run in export mode.
     #[clap(long, hide = true)]
-    export: bool,
+    enable_export_mode: bool,
 
     /// Use presentation mode.
     #[clap(short, long, default_value_t = false)]
@@ -59,6 +65,10 @@ struct Cli {
     #[clap(long)]
     validate_overflows: bool,
 
+    /// Enable code snippet execution.
+    #[clap(short = 'x', long)]
+    enable_snippet_execution: bool,
+
     /// The path to the configuration file.
     #[clap(short, long)]
     config_file: Option<String>,
@@ -78,7 +88,14 @@ fn create_splash() -> String {
     )
 }
 
-fn load_customizations(config_file_path: Option<PathBuf>) -> Result<(Config, Themes), Box<dyn std::error::Error>> {
+#[derive(Default)]
+struct Customizations {
+    config: Config,
+    themes: Themes,
+    code_executor: SnippetExecutor,
+}
+
+fn load_customizations(config_file_path: Option<PathBuf>) -> Result<Customizations, Box<dyn std::error::Error>> {
     let configs_path: PathBuf = match env::var("XDG_CONFIG_HOME") {
         Ok(path) => Path::new(&path).join("presenterm"),
         Err(_) => {
@@ -91,20 +108,18 @@ fn load_customizations(config_file_path: Option<PathBuf>) -> Result<(Config, The
     let themes = load_themes(&configs_path)?;
     let config_file_path = config_file_path.unwrap_or_else(|| configs_path.join("config.yaml"));
     let config = Config::load(&config_file_path)?;
-    Ok((config, themes))
+    let code_executor = SnippetExecutor::new(config.snippet.exec.custom.clone())?;
+    Ok(Customizations { config, themes, code_executor })
 }
 
 fn load_themes(config_path: &Path) -> Result<Themes, Box<dyn std::error::Error>> {
     let themes_path = config_path.join("themes");
 
     let mut highlight_themes = HighlightThemeSet::default();
-    highlight_themes.register_from_directory(&themes_path.join("highlighting"))?;
+    highlight_themes.register_from_directory(themes_path.join("highlighting"))?;
 
     let mut presentation_themes = PresentationThemeSet::default();
-    let register_result = presentation_themes.register_from_directory(&themes_path);
-    if let Err(e @ (LoadThemeError::Duplicate(_) | LoadThemeError::Corrupted(..))) = register_result {
-        return Err(e.into());
-    }
+    presentation_themes.register_from_directory(&themes_path)?;
 
     let themes = Themes { presentation: presentation_themes, highlight: highlight_themes };
     Ok(themes)
@@ -120,11 +135,13 @@ fn make_builder_options(config: &Config, mode: &PresentMode, force_default_theme
         allow_mutations: !matches!(mode, PresentMode::Export),
         implicit_slide_ends: config.options.implicit_slide_ends.unwrap_or_default(),
         command_prefix: config.options.command_prefix.clone().unwrap_or_default(),
+        image_attribute_prefix: config.options.image_attributes_prefix.clone().unwrap_or_else(|| "image:".to_string()),
         incremental_lists: config.options.incremental_lists.unwrap_or_default(),
         force_default_theme,
         end_slide_shorthand: config.options.end_slide_shorthand.unwrap_or_default(),
         print_modal_background: false,
         strict_front_matter_parsing: config.options.strict_front_matter_parsing.unwrap_or(true),
+        enable_snippet_execution: config.snippet.exec.enable,
     }
 }
 
@@ -140,7 +157,7 @@ fn load_default_theme(config: &Config, themes: &Themes, cli: &Cli) -> Presentati
 }
 
 fn select_graphics_mode(cli: &Cli, config: &Config) -> GraphicsMode {
-    if cli.export || cli.export_pdf || cli.generate_pdf_metadata {
+    if cli.enable_export_mode || cli.export_pdf || cli.generate_pdf_metadata {
         GraphicsMode::AsciiBlocks
     } else {
         let protocol = cli.image_protocol.as_ref().unwrap_or(&config.defaults.image_protocol);
@@ -164,11 +181,18 @@ fn overflow_validation(mode: &PresentMode, config: &ValidateOverflows) -> bool {
 }
 
 fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
-    let (config, themes) = load_customizations(cli.config_file.clone().map(PathBuf::from))?;
+    if cli.generate_config_file_schema {
+        let schema = schemars::schema_for!(Config);
+        serde_json::to_writer_pretty(io::stdout(), &schema).map_err(|e| format!("failed to write schema: {e}"))?;
+        return Ok(());
+    }
+
+    let Customizations { config, themes, code_executor } =
+        load_customizations(cli.config_file.clone().map(PathBuf::from))?;
 
     let default_theme = load_default_theme(&config, &themes, &cli);
     let force_default_theme = cli.theme.is_some();
-    let mode = match (cli.present, cli.export) {
+    let mode = match (cli.present, cli.enable_export_mode) {
         (true, _) => PresentMode::Presentation,
         (false, true) => PresentMode::Export,
         (false, false) => PresentMode::Development,
@@ -191,13 +215,23 @@ fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
     let validate_overflows = overflow_validation(&mode, &config.defaults.validate_overflows) || cli.validate_overflows;
     let resources_path = path.parent().unwrap_or(Path::new("/"));
     let mut options = make_builder_options(&config, &mode, force_default_theme);
+    if cli.enable_snippet_execution {
+        options.enable_snippet_execution = true;
+    }
     let graphics_mode = select_graphics_mode(&cli, &config);
-    let printer = Rc::new(ImagePrinter::new(graphics_mode.clone())?);
+    let printer = Arc::new(ImagePrinter::new(graphics_mode.clone())?);
     let registry = ImageRegistry(printer.clone());
     let resources = Resources::new(resources_path, registry.clone());
-    let typst = TypstRender::new(config.typst.ppi, registry);
+    let third_party_config = ThirdPartyConfigs {
+        typst_ppi: config.typst.ppi.to_string(),
+        mermaid_scale: config.mermaid.scale.to_string(),
+        threads: config.snippet.render.threads,
+    };
+    let third_party = ThirdPartyRender::new(third_party_config, registry, resources_path);
+    let code_executor = Rc::new(code_executor);
     if cli.export_pdf || cli.generate_pdf_metadata {
-        let mut exporter = Exporter::new(parser, &default_theme, resources, typst, themes, options);
+        let mut exporter =
+            Exporter::new(parser, &default_theme, resources, third_party, code_executor, themes, options);
         let mut args = Vec::new();
         if let Some(theme) = cli.theme.as_ref() {
             args.extend(["--theme", theme]);
@@ -205,6 +239,9 @@ fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
         if let Some(path) = cli.config_file.as_ref() {
             args.extend(["--config-file", path]);
         }
+        if cli.enable_snippet_execution {
+            args.push("-x");
+        }
         if cli.export_pdf {
             exporter.export_pdf(&path, &args)?;
         } else {
@@ -222,7 +259,17 @@ fn run(mut cli: Cli) -> Result<(), Box<dyn std::error::Error>> {
             bindings: config.bindings,
             validate_overflows,
         };
-        let presenter = Presenter::new(&default_theme, commands, parser, resources, typst, themes, printer, options);
+        let presenter = Presenter::new(
+            &default_theme,
+            commands,
+            parser,
+            resources,
+            third_party,
+            code_executor,
+            themes,
+            printer,
+            options,
+        );
         presenter.present(&path)?;
     }
     Ok(())
