@@ -1,4 +1,6 @@
-use super::elements::{Code, CodeAttributes, CodeLanguage, Highlight, HighlightGroup};
+use super::elements::{
+    Highlight, HighlightGroup, Percent, PercentParseError, Snippet, SnippetAttributes, SnippetLanguage,
+};
 use comrak::nodes::NodeCodeBlock;
 use strum::EnumDiscriminants;
 
@@ -7,86 +9,34 @@ pub(crate) type ParseResult<T> = Result<T, CodeBlockParseError>;
 pub(crate) struct CodeBlockParser;
 
 impl CodeBlockParser {
-    pub(crate) fn parse(code_block: &NodeCodeBlock) -> ParseResult<Code> {
+    pub(crate) fn parse(code_block: &NodeCodeBlock) -> ParseResult<Snippet> {
         let (language, attributes) = Self::parse_block_info(&code_block.info)?;
-        let code = Code { contents: code_block.literal.clone(), language, attributes };
+        let code = Snippet { contents: code_block.literal.clone(), language, attributes };
         Ok(code)
     }
 
-    fn parse_block_info(input: &str) -> ParseResult<(CodeLanguage, CodeAttributes)> {
+    fn parse_block_info(input: &str) -> ParseResult<(SnippetLanguage, SnippetAttributes)> {
         let (language, input) = Self::parse_language(input);
         let attributes = Self::parse_attributes(input)?;
-        if attributes.execute && !language.supports_execution() {
-            return Err(CodeBlockParseError::UnsupportedAttribute(language, "execution"));
-        }
         if attributes.auto_render && !language.supports_auto_render() {
             return Err(CodeBlockParseError::UnsupportedAttribute(language, "rendering"));
         }
+        if attributes.width.is_some() && !attributes.auto_render {
+            return Err(CodeBlockParseError::NotRenderSnippet("width"));
+        }
         Ok((language, attributes))
     }
 
-    fn parse_language(input: &str) -> (CodeLanguage, &str) {
+    fn parse_language(input: &str) -> (SnippetLanguage, &str) {
         let token = Self::next_identifier(input);
-        use CodeLanguage::*;
-        let language = match token {
-            "ada" => Ada,
-            "asp" => Asp,
-            "awk" => Awk,
-            "c" => C,
-            "cmake" => CMake,
-            "crontab" => Crontab,
-            "csharp" => CSharp,
-            "clojure" => Clojure,
-            "cpp" | "c++" => Cpp,
-            "css" => Css,
-            "d" => DLang,
-            "diff" => Diff,
-            "docker" => Docker,
-            "dotenv" => Dotenv,
-            "elixir" => Elixir,
-            "elm" => Elm,
-            "erlang" => Erlang,
-            "go" => Go,
-            "haskell" => Haskell,
-            "html" => Html,
-            "java" => Java,
-            "javascript" | "js" => JavaScript,
-            "json" => Json,
-            "kotlin" => Kotlin,
-            "latex" => Latex,
-            "lua" => Lua,
-            "make" => Makefile,
-            "markdown" => Markdown,
-            "nix" => Nix,
-            "ocaml" => OCaml,
-            "perl" => Perl,
-            "php" => Php,
-            "protobuf" => Protobuf,
-            "puppet" => Puppet,
-            "python" => Python,
-            "r" => R,
-            "rust" => Rust,
-            "scala" => Scala,
-            "shell" => Shell("sh".into()),
-            interpreter @ ("bash" | "sh" | "zsh" | "fish") => Shell(interpreter.into()),
-            "sql" => Sql,
-            "svelte" => Svelte,
-            "swift" => Swift,
-            "terraform" => Terraform,
-            "typescript" | "ts" => TypeScript,
-            "typst" => Typst,
-            "xml" => Xml,
-            "yaml" => Yaml,
-            "vue" => Vue,
-            "zig" => Zig,
-            _ => Unknown,
-        };
+        // this always returns `Ok` given we fall back to `Unknown` if we don't know the language.
+        let language = token.parse().expect("language parsing");
         let rest = &input[token.len()..];
         (language, rest)
     }
 
-    fn parse_attributes(mut input: &str) -> ParseResult<CodeAttributes> {
-        let mut attributes = CodeAttributes::default();
+    fn parse_attributes(mut input: &str) -> ParseResult<SnippetAttributes> {
+        let mut attributes = SnippetAttributes::default();
         let mut processed_attributes = Vec::new();
         while let (Some(attribute), rest) = Self::parse_attribute(input)? {
             let discriminant = AttributeDiscriminants::from(&attribute);
@@ -98,6 +48,7 @@ impl CodeBlockParser {
                 Attribute::Exec => attributes.execute = true,
                 Attribute::AutoRender => attributes.auto_render = true,
                 Attribute::HighlightedLines(lines) => attributes.highlight_groups = lines,
+                Attribute::Width(width) => attributes.width = Some(width),
             };
             processed_attributes.push(discriminant);
             input = rest;
@@ -117,6 +68,11 @@ impl CodeBlockParser {
                     "line_numbers" => Attribute::LineNumbers,
                     "exec" => Attribute::Exec,
                     "render" => Attribute::AutoRender,
+                    token if token.starts_with("width:") => {
+                        let value = input.split_once("+width:").unwrap().1;
+                        let (width, input) = Self::parse_width(value)?;
+                        return Ok((Some(Attribute::Width(width)), input));
+                    }
                     _ => return Err(CodeBlockParseError::InvalidToken(Self::next_identifier(input).into())),
                 };
                 (Some(attribute), &input[token.len() + 1..])
@@ -182,6 +138,12 @@ impl CodeBlockParser {
             .map_err(|_| CodeBlockParseError::InvalidHighlightedLines(format!("not a number: '{input}'")))
     }
 
+    fn parse_width(input: &str) -> ParseResult<(Percent, &str)> {
+        let end_index = input.find(' ').unwrap_or(input.len());
+        let value = input[0..end_index].parse().map_err(CodeBlockParseError::InvalidWidth)?;
+        Ok((value, &input[end_index..]))
+    }
+
     fn skip_whitespace(input: &str) -> &str {
         input.trim_start_matches(' ')
     }
@@ -202,11 +164,17 @@ pub(crate) enum CodeBlockParseError {
     #[error("invalid highlighted lines: {0}")]
     InvalidHighlightedLines(String),
 
+    #[error("invalid width: {0}")]
+    InvalidWidth(PercentParseError),
+
     #[error("duplicate attribute: {0}")]
     DuplicateAttribute(&'static str),
 
     #[error("language {0:?} does not support {1}")]
-    UnsupportedAttribute(CodeLanguage, &'static str),
+    UnsupportedAttribute(SnippetLanguage, &'static str),
+
+    #[error("attribute {0} can only be set in +render blocks")]
+    NotRenderSnippet(&'static str),
 }
 
 #[derive(EnumDiscriminants)]
@@ -215,6 +183,7 @@ enum Attribute {
     Exec,
     AutoRender,
     HighlightedLines(Vec<HighlightGroup>),
+    Width(Percent),
 }
 
 #[cfg(test)]
@@ -223,24 +192,28 @@ mod test {
     use rstest::rstest;
     use Highlight::*;
 
-    fn parse_language(input: &str) -> CodeLanguage {
+    fn parse_language(input: &str) -> SnippetLanguage {
         let (language, _) = CodeBlockParser::parse_block_info(input).expect("parse failed");
         language
     }
 
-    fn parse_attributes(input: &str) -> CodeAttributes {
-        let (_, attributes) = CodeBlockParser::parse_block_info(input).expect("parse failed");
-        attributes
+    fn try_parse_attributes(input: &str) -> Result<SnippetAttributes, CodeBlockParseError> {
+        let (_, attributes) = CodeBlockParser::parse_block_info(input)?;
+        Ok(attributes)
+    }
+
+    fn parse_attributes(input: &str) -> SnippetAttributes {
+        try_parse_attributes(input).expect("parse failed")
     }
 
     #[test]
     fn unknown_language() {
-        assert_eq!(parse_language("potato"), CodeLanguage::Unknown);
+        assert_eq!(parse_language("potato"), SnippetLanguage::Unknown("potato".to_string()));
     }
 
     #[test]
     fn no_attributes() {
-        assert_eq!(parse_language("rust"), CodeLanguage::Rust);
+        assert_eq!(parse_language("rust"), SnippetLanguage::Rust);
     }
 
     #[test]
@@ -307,4 +280,18 @@ mod test {
         assert_eq!(attributes.highlight_groups[0], HighlightGroup::new(vec![Range(1..4), Single(5)]));
         assert_eq!(attributes.highlight_groups[1], HighlightGroup::new(vec![Range(6..10)]));
     }
+
+    #[test]
+    fn parse_width() {
+        let attributes = parse_attributes("mermaid +width:50% +render");
+        assert!(attributes.auto_render);
+        assert_eq!(attributes.width, Some(Percent(50)));
+    }
+
+    #[test]
+    fn invalid_width() {
+        try_parse_attributes("mermaid +width:50%% +render").expect_err("parse succeeded");
+        try_parse_attributes("mermaid +width: +render").expect_err("parse succeeded");
+        try_parse_attributes("mermaid +width:50%").expect_err("parse succeeded");
+    }
 }
