@@ -3,7 +3,7 @@ use crate::style::Color;
 use base64::{engine::general_purpose::STANDARD, Engine};
 use console::{Key, Term};
 use crossterm::{cursor::MoveToColumn, style::SetForegroundColor, QueueableCommand};
-use image::{codecs::gif::GifDecoder, io::Reader, AnimationDecoder, Delay, DynamicImage, EncodableLayout, RgbaImage};
+use image::{codecs::gif::GifDecoder, AnimationDecoder, Delay, DynamicImage, EncodableLayout, ImageReader, RgbaImage};
 use rand::Rng;
 use std::{
     fmt,
@@ -11,7 +11,6 @@ use std::{
     io::{self, BufReader, Write},
     path::{Path, PathBuf},
     sync::atomic::{AtomicU32, Ordering},
-    u8,
 };
 use tempfile::{tempdir, NamedTempFile, TempDir};
 
@@ -352,7 +351,7 @@ impl KittyPrinter {
     fn load_raw_resource(path: &Path) -> Result<RawResource, RegisterImageError> {
         let file = File::open(path)?;
         if path.extension().unwrap_or_default() == "gif" {
-            let decoder = GifDecoder::new(file)?;
+            let decoder = GifDecoder::new(BufReader::new(file))?;
             let mut frames = Vec::new();
             for frame in decoder.into_frames() {
                 let frame = frame?;
@@ -361,7 +360,7 @@ impl KittyPrinter {
             }
             Ok(RawResource::Gif(frames))
         } else {
-            let reader = Reader::new(BufReader::new(file)).with_guessed_format()?;
+            let reader = ImageReader::new(BufReader::new(file)).with_guessed_format()?;
             let image = reader.decode()?;
             Ok(RawResource::Image(image.into_rgba8()))
         }
