@@ -7,7 +7,14 @@ use crate::{
     theme::{Alignment, Margin, PresentationTheme},
 };
 use serde::Deserialize;
-use std::{cell::RefCell, fmt::Debug, ops::Deref, rc::Rc};
+use std::{
+    cell::RefCell,
+    collections::HashSet,
+    fmt::Debug,
+    ops::Deref,
+    rc::Rc,
+    sync::{Arc, Mutex},
+};
 
 #[derive(Debug)]
 pub(crate) struct Modals {
@@ -20,7 +27,7 @@ pub(crate) struct Modals {
 pub(crate) struct Presentation {
     slides: Vec<Slide>,
     modals: Modals,
-    state: PresentationState,
+    pub(crate) state: PresentationState,
 }
 
 impl Presentation {
@@ -66,14 +73,17 @@ impl Presentation {
         if current_slide.move_next() {
             return true;
         }
-        let current_slide_index = self.current_slide_index();
-        if current_slide_index < self.slides.len() - 1 {
-            self.state.set_current_slide_index(current_slide_index + 1);
-            // Going forward we show only the first chunk.
-            self.current_slide_mut().show_first_chunk();
-            true
+        self.jump_next_slide()
+    }
+
+    /// Show all chunks in this slide, or jump to the next if already applied.
+    pub(crate) fn jump_next_fast(&mut self) -> bool {
+        let current_slide = self.current_slide_mut();
+        if current_slide.visible_chunks == current_slide.chunks.len() {
+            self.jump_next_slide()
         } else {
-            false
+            current_slide.show_all_chunks();
+            true
         }
     }
 
@@ -83,14 +93,17 @@ impl Presentation {
         if current_slide.move_previous() {
             return true;
         }
-        let current_slide_index = self.current_slide_index();
-        if current_slide_index > 0 {
-            self.state.set_current_slide_index(current_slide_index - 1);
-            // Going backwards we show all chunks.
-            self.current_slide_mut().show_all_chunks();
+        self.jump_previous_slide()
+    }
+
+    /// Show only the first chunk in this slide or jump to the previous slide if already there.
+    pub(crate) fn jump_previous_fast(&mut self) -> bool {
+        let current_slide = self.current_slide_mut();
+        if current_slide.visible_chunks == current_slide.chunks.len() && current_slide.chunks.len() > 1 {
+            current_slide.show_first_chunk();
             true
         } else {
-            false
+            self.jump_previous_slide()
         }
     }
 
@@ -127,28 +140,73 @@ impl Presentation {
         self.current_slide().current_chunk_index()
     }
 
-    /// Render all widgets in this slide.
-    pub(crate) fn render_slide_widgets(&mut self) -> bool {
+    /// Trigger async render operations in all slides.
+    pub(crate) fn trigger_all_async_renders(&mut self) -> HashSet<usize> {
+        let mut triggered_slides = HashSet::new();
+        for (index, slide) in self.slides.iter_mut().enumerate() {
+            for operation in slide.iter_operations_mut() {
+                if let RenderOperation::RenderAsync(operation) = operation {
+                    if operation.start_render() {
+                        triggered_slides.insert(index);
+                    }
+                }
+            }
+        }
+        triggered_slides
+    }
+
+    /// Trigger async render operations in this slide.
+    pub(crate) fn trigger_slide_async_renders(&mut self) -> bool {
         let slide = self.current_slide_mut();
         let mut any_rendered = false;
-        for operation in slide.iter_operations_mut() {
-            if let RenderOperation::RenderOnDemand(operation) = operation {
-                any_rendered = any_rendered || operation.start_render();
+        for operation in slide.iter_visible_operations_mut() {
+            if let RenderOperation::RenderAsync(operation) = operation {
+                let is_rendered = operation.start_render();
+                any_rendered = any_rendered || is_rendered;
             }
         }
         any_rendered
     }
 
-    /// Poll every widget in the current slide and check whether they're rendered.
-    pub(crate) fn widgets_rendered(&mut self) -> bool {
-        let slide = self.current_slide_mut();
-        let mut all_rendered = true;
-        for operation in slide.iter_operations_mut() {
-            if let RenderOperation::RenderOnDemand(operation) = operation {
-                all_rendered = all_rendered && matches!(operation.poll_state(), RenderOnDemandState::Rendered);
+    // Get all slides that contain async render operations.
+    pub(crate) fn slides_with_async_renders(&self) -> HashSet<usize> {
+        let mut indexes = HashSet::new();
+        for (index, slide) in self.slides.iter().enumerate() {
+            for operation in slide.iter_operations() {
+                if let RenderOperation::RenderAsync(operation) = operation {
+                    if matches!(operation.poll_state(), RenderAsyncState::Rendering { .. }) {
+                        indexes.insert(index);
+                        break;
+                    }
+                }
             }
         }
-        all_rendered
+        indexes
+    }
+
+    /// Poll every async render operation in the current slide and check whether they're completed.
+    pub(crate) fn poll_slide_async_renders(&mut self) -> RenderAsyncState {
+        let slide = self.current_slide_mut();
+        let mut slide_state = RenderAsyncState::Rendered;
+        for operation in slide.iter_operations_mut() {
+            if let RenderOperation::RenderAsync(operation) = operation {
+                let state = operation.poll_state();
+                slide_state = match (&slide_state, &state) {
+                    // If one finished rendering and another one still is rendering, claim that we
+                    // are still rendering and there's modifications.
+                    (RenderAsyncState::JustFinishedRendering, RenderAsyncState::Rendering { .. })
+                    | (RenderAsyncState::Rendering { .. }, RenderAsyncState::JustFinishedRendering) => {
+                        RenderAsyncState::Rendering { modified: true }
+                    }
+                    // Render + modified overrides anything, rendering overrides only "rendered".
+                    (_, RenderAsyncState::Rendering { modified: true })
+                    | (RenderAsyncState::Rendered, RenderAsyncState::Rendering { .. })
+                    | (_, RenderAsyncState::JustFinishedRendering) => state,
+                    _ => slide_state,
+                };
+            }
+        }
+        slide_state
     }
 
     /// Run a callback through every operation and let it mutate it in place.
@@ -171,6 +229,30 @@ impl Presentation {
         let index = self.current_slide_index();
         &mut self.slides[index]
     }
+
+    fn jump_next_slide(&mut self) -> bool {
+        let current_slide_index = self.current_slide_index();
+        if current_slide_index < self.slides.len() - 1 {
+            self.state.set_current_slide_index(current_slide_index + 1);
+            // Going forward we show only the first chunk.
+            self.current_slide_mut().show_first_chunk();
+            true
+        } else {
+            false
+        }
+    }
+
+    fn jump_previous_slide(&mut self) -> bool {
+        let current_slide_index = self.current_slide_index();
+        if current_slide_index > 0 {
+            self.state.set_current_slide_index(current_slide_index - 1);
+            // Going backwards we show all chunks.
+            self.current_slide_mut().show_all_chunks();
+            true
+        } else {
+            false
+        }
+    }
 }
 
 impl From<Vec<Slide>> for Presentation {
@@ -180,9 +262,18 @@ impl From<Vec<Slide>> for Presentation {
     }
 }
 
+#[derive(Debug)]
+pub(crate) struct AsyncPresentationError {
+    pub(crate) slide: usize,
+    pub(crate) error: String,
+}
+
+pub(crate) type AsyncPresentationErrorHolder = Arc<Mutex<Option<AsyncPresentationError>>>;
+
 #[derive(Debug, Default)]
 pub(crate) struct PresentationStateInner {
     current_slide_index: usize,
+    async_error_holder: AsyncPresentationErrorHolder,
 }
 
 #[derive(Clone, Debug, Default)]
@@ -191,6 +282,10 @@ pub(crate) struct PresentationState {
 }
 
 impl PresentationState {
+    pub(crate) fn async_error_holder(&self) -> AsyncPresentationErrorHolder {
+        self.inner.deref().borrow().async_error_holder.clone()
+    }
+
     pub(crate) fn current_slide_index(&self) -> usize {
         self.inner.deref().borrow().current_slide_index
     }
@@ -240,10 +335,18 @@ impl Slide {
     }
 
     pub(crate) fn iter_operations(&self) -> impl Iterator<Item = &RenderOperation> + Clone {
-        self.chunks.iter().take(self.visible_chunks).flat_map(|chunk| chunk.operations.iter()).chain(self.footer.iter())
+        self.chunks.iter().flat_map(|chunk| chunk.operations.iter()).chain(self.footer.iter())
     }
 
     pub(crate) fn iter_operations_mut(&mut self) -> impl Iterator<Item = &mut RenderOperation> {
+        self.chunks.iter_mut().flat_map(|chunk| chunk.operations.iter_mut()).chain(self.footer.iter_mut())
+    }
+
+    pub(crate) fn iter_visible_operations(&self) -> impl Iterator<Item = &RenderOperation> + Clone {
+        self.chunks.iter().take(self.visible_chunks).flat_map(|chunk| chunk.operations.iter()).chain(self.footer.iter())
+    }
+
+    pub(crate) fn iter_visible_operations_mut(&mut self) -> impl Iterator<Item = &mut RenderOperation> {
         self.chunks
             .iter_mut()
             .take(self.visible_chunks)
@@ -257,11 +360,11 @@ impl Slide {
 
     #[cfg(test)]
     pub(crate) fn into_operations(self) -> Vec<RenderOperation> {
-        self.chunks.into_iter().flat_map(|chunk| chunk.operations.into_iter()).chain(self.footer.into_iter()).collect()
+        self.chunks.into_iter().flat_map(|chunk| chunk.operations.into_iter()).chain(self.footer).collect()
     }
 
     fn jump_chunk(&mut self, chunk_index: usize) {
-        self.visible_chunks = (chunk_index + 1).min(self.chunks.len());
+        self.visible_chunks = chunk_index.saturating_add(1).min(self.chunks.len());
         for chunk in self.chunks.iter().take(self.visible_chunks - 1) {
             chunk.apply_all_mutations();
         }
@@ -394,6 +497,10 @@ pub(crate) struct PresentationMetadata {
     #[serde(default)]
     pub(crate) author: Option<String>,
 
+    /// The presentation authors.
+    #[serde(default)]
+    pub(crate) authors: Vec<String>,
+
     /// The presentation's theme metadata.
     #[serde(default)]
     pub(crate) theme: PresentationThemeMetadata,
@@ -421,13 +528,19 @@ pub(crate) struct PresentationThemeMetadata {
 
 /// A line of preformatted text to be rendered.
 #[derive(Clone, Debug, PartialEq)]
-pub(crate) struct PreformattedLine {
-    pub(crate) text: String,
+pub(crate) struct BlockLine {
+    pub(crate) text: BlockLineText,
     pub(crate) unformatted_length: u16,
     pub(crate) block_length: u16,
     pub(crate) alignment: Alignment,
 }
 
+#[derive(Clone, Debug, PartialEq)]
+pub(crate) enum BlockLineText {
+    Preformatted(String),
+    Weighted(WeightedTextBlock),
+}
+
 /// A render operation.
 ///
 /// Render operations are primitives that allow the input markdown file to be decoupled with what
@@ -462,11 +575,8 @@ pub(crate) enum RenderOperation {
     /// Render an image.
     RenderImage(Image, ImageProperties),
 
-    /// Render a preformatted line.
-    ///
-    /// The line will usually already have terminal escape codes that include colors and formatting
-    /// embedded in it.
-    RenderPreformattedLine(PreformattedLine),
+    /// Render a line.
+    RenderBlockLine(BlockLine),
 
     /// Render a dynamically generated sequence of render operations.
     ///
@@ -475,8 +585,8 @@ pub(crate) enum RenderOperation {
     /// [RenderOperation] with the screen itself.
     RenderDynamic(Rc<dyn AsRenderOperations>),
 
-    /// An operation that is rendered on demand.
-    RenderOnDemand(Rc<dyn RenderOnDemand>),
+    /// An operation that is rendered asynchronously.
+    RenderAsync(Rc<dyn RenderAsync>),
 
     /// Initialize a column layout.
     ///
@@ -500,7 +610,7 @@ pub(crate) enum RenderOperation {
 }
 
 /// The properties of an image being rendered.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, PartialEq)]
 pub(crate) struct ImageProperties {
     pub(crate) z_index: i32,
     pub(crate) size: ImageSize,
@@ -509,11 +619,14 @@ pub(crate) struct ImageProperties {
 }
 
 /// The size used when printing an image.
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug, Default, PartialEq)]
 pub(crate) enum ImageSize {
     #[default]
-    Scaled,
+    ShrinkIfNeeded,
     Specific(u16, u16),
+    WidthScaled {
+        ratio: f64,
+    },
 }
 
 /// Slide properties, set on initialization.
@@ -535,37 +648,44 @@ pub(crate) trait AsRenderOperations: Debug + 'static {
     fn diffable_content(&self) -> Option<&str>;
 }
 
-/// A type that can be rendered on demand.
-pub(crate) trait RenderOnDemand: AsRenderOperations {
-    /// Start the on demand render for this operation.
+/// An operation that can be rendered asynchronously.
+pub(crate) trait RenderAsync: AsRenderOperations {
+    /// Start the render for this operation.
+    ///
+    /// Should return true if the invocation triggered the rendering (aka if rendering wasn't
+    /// already started before).
     fn start_render(&self) -> bool;
 
-    /// Poll and update the internal on demand state and return the latest.
-    fn poll_state(&self) -> RenderOnDemandState;
+    /// Update the internal state and return the updated state.
+    fn poll_state(&self) -> RenderAsyncState;
 }
 
-/// The state of a [RenderOnDemand].
+/// The state of a [RenderAsync].
 #[derive(Clone, Debug, Default)]
-pub(crate) enum RenderOnDemandState {
+pub(crate) enum RenderAsyncState {
     #[default]
     NotStarted,
-    Rendering,
+    Rendering {
+        modified: bool,
+    },
     Rendered,
+    JustFinishedRendering,
 }
 
 #[cfg(test)]
 mod test {
-    use std::cell::RefCell;
-
     use super::*;
     use rstest::rstest;
+    use std::cell::RefCell;
 
     #[derive(Clone)]
     enum Jump {
         First,
         Last,
         Next,
+        NextFast,
         Previous,
+        PreviousFast,
         Specific(usize),
     }
 
@@ -576,7 +696,9 @@ mod test {
                 First => presentation.jump_first_slide(),
                 Last => presentation.jump_last_slide(),
                 Next => presentation.jump_next(),
+                NextFast => presentation.jump_next_fast(),
                 Previous => presentation.jump_previous(),
+                PreviousFast => presentation.jump_previous_fast(),
                 Specific(index) => presentation.go_to_slide(*index),
             };
         }
@@ -635,9 +757,14 @@ mod test {
     #[rstest]
     #[case::previous_from_first(0, &[Jump::Previous], 0, 0)]
     #[case::next_from_first(0, &[Jump::Next], 0, 1)]
-    #[case::next_next_from_first(0, &[Jump::Next, Jump::Next], 1, 0)]
+    #[case::next_next_from_first(0, &[Jump::Next, Jump::Next], 0, 2)]
+    #[case::next_next_next_from_first(0, &[Jump::Next, Jump::Next, Jump::Next], 1, 0)]
+    #[case::next_fast_from_first(0, &[Jump::NextFast], 0, 2)]
+    #[case::next_fast_twice_from_first(0, &[Jump::NextFast, Jump::NextFast], 1, 0)]
     #[case::last_from_first(0, &[Jump::Last], 2, 0)]
-    #[case::previous_from_second(1, &[Jump::Previous], 0, 1)]
+    #[case::previous_from_second(1, &[Jump::Previous], 0, 2)]
+    #[case::previous_fast_from_second(1, &[Jump::PreviousFast], 0, 2)]
+    #[case::previous_fast_twice_from_second(1, &[Jump::PreviousFast, Jump::PreviousFast], 0, 0)]
     #[case::next_from_second(1, &[Jump::Next], 1, 1)]
     #[case::specific_first_from_second(1, &[Jump::Specific(0)], 0, 0)]
     #[case::specific_last_from_second(1, &[Jump::Specific(2)], 2, 0)]
@@ -649,9 +776,9 @@ mod test {
         #[case] expected_chunk: usize,
     ) {
         let mut presentation = Presentation::from(vec![
-            Slide::new(vec![SlideChunk::from(SlideChunk::default()), SlideChunk::default()], vec![]),
-            Slide::new(vec![SlideChunk::from(SlideChunk::default()), SlideChunk::default()], vec![]),
-            Slide::new(vec![SlideChunk::from(SlideChunk::default()), SlideChunk::default()], vec![]),
+            Slide::new(vec![SlideChunk::default(), SlideChunk::default(), SlideChunk::default()], vec![]),
+            Slide::new(vec![SlideChunk::default(), SlideChunk::default()], vec![]),
+            Slide::new(vec![SlideChunk::default(), SlideChunk::default()], vec![]),
         ]);
         presentation.go_to_slide(from);
 
@@ -688,18 +815,12 @@ mod test {
         let mut presentation = Presentation::from(vec![
             SlideBuilder::default()
                 .chunks(vec![
-                    SlideChunk::from(SlideChunk::new(
-                        vec![],
-                        vec![Box::new(DummyMutator::new(1)), Box::new(DummyMutator::new(2))],
-                    )),
+                    SlideChunk::new(vec![], vec![Box::new(DummyMutator::new(1)), Box::new(DummyMutator::new(2))]),
                     SlideChunk::default(),
                 ])
                 .build(),
             SlideBuilder::default()
-                .chunks(vec![
-                    SlideChunk::from(SlideChunk::new(vec![], vec![Box::new(DummyMutator::new(2))])),
-                    SlideChunk::default(),
-                ])
+                .chunks(vec![SlideChunk::new(vec![], vec![Box::new(DummyMutator::new(2))]), SlideChunk::default()])
                 .build(),
         ]);
         presentation.go_to_slide(from);
