@@ -1,20 +1,21 @@
 use crate::{
     custom::KeyBindingsConfig,
     diff::PresentationDiffer,
+    execute::SnippetExecutor,
     export::ImageReplacer,
     input::source::{Command, CommandSource},
     markdown::parse::{MarkdownParser, ParseError},
     media::{printer::ImagePrinter, register::ImageRegistry},
-    presentation::Presentation,
+    presentation::{Presentation, RenderAsyncState},
     processing::builder::{BuildError, PresentationBuilder, PresentationBuilderOptions, Themes},
     render::{
-        draw::{RenderError, RenderResult, TerminalDrawer},
+        draw::{ErrorSource, RenderError, RenderResult, TerminalDrawer},
         properties::WindowSize,
         validate::OverflowValidator,
     },
     resource::Resources,
     theme::PresentationTheme,
-    typst::TypstRender,
+    third_party::ThirdPartyRender,
 };
 use std::{
     collections::HashSet,
@@ -22,8 +23,10 @@ use std::{
     fs,
     io::{self, Stdout},
     mem,
+    ops::Deref,
     path::Path,
     rc::Rc,
+    sync::Arc,
 };
 
 pub struct PresenterOptions {
@@ -42,10 +45,11 @@ pub struct Presenter<'a> {
     commands: CommandSource,
     parser: MarkdownParser<'a>,
     resources: Resources,
-    typst: TypstRender,
+    third_party: ThirdPartyRender,
+    code_executor: Rc<SnippetExecutor>,
     state: PresenterState,
-    slides_with_pending_widgets: HashSet<usize>,
-    image_printer: Rc<ImagePrinter>,
+    slides_with_pending_async_renders: HashSet<usize>,
+    image_printer: Arc<ImagePrinter>,
     themes: Themes,
     options: PresenterOptions,
 }
@@ -58,9 +62,10 @@ impl<'a> Presenter<'a> {
         commands: CommandSource,
         parser: MarkdownParser<'a>,
         resources: Resources,
-        typst: TypstRender,
+        third_party: ThirdPartyRender,
+        code_executor: Rc<SnippetExecutor>,
         themes: Themes,
-        image_printer: Rc<ImagePrinter>,
+        image_printer: Arc<ImagePrinter>,
         options: PresenterOptions,
     ) -> Self {
         Self {
@@ -68,9 +73,10 @@ impl<'a> Presenter<'a> {
             commands,
             parser,
             resources,
-            typst,
+            third_party,
+            code_executor,
             state: PresenterState::Empty,
-            slides_with_pending_widgets: HashSet::new(),
+            slides_with_pending_async_renders: HashSet::new(),
             image_printer,
             themes,
             options,
@@ -85,12 +91,23 @@ impl<'a> Presenter<'a> {
         let mut drawer =
             TerminalDrawer::new(io::stdout(), self.image_printer.clone(), self.options.font_size_fallback)?;
         loop {
+            if matches!(self.options.mode, PresentMode::Export) {
+                if let PresenterState::Failure { error, .. } = &self.state {
+                    return Err(PresentationError::Fatal(format!("failed to run presentation: {error}")));
+                }
+            }
+            // Poll async renders once before we draw just in case.
+            self.poll_async_renders()?;
             self.render(&mut drawer)?;
-            self.update_widgets(&mut drawer)?;
 
             loop {
-                self.update_widgets(&mut drawer)?;
+                if self.poll_async_renders()? {
+                    self.render(&mut drawer)?;
+                }
                 let Some(command) = self.commands.try_next_command()? else {
+                    if self.check_async_error() {
+                        break;
+                    }
                     continue;
                 };
                 match self.apply_command(command) {
@@ -102,26 +119,49 @@ impl<'a> Presenter<'a> {
                     CommandSideEffect::Redraw => {
                         break;
                     }
-                    CommandSideEffect::PollWidgets => {
-                        self.slides_with_pending_widgets.insert(self.state.presentation().current_slide_index());
-                    }
                     CommandSideEffect::None => (),
                 };
             }
         }
     }
 
-    fn update_widgets(&mut self, drawer: &mut TerminalDrawer<Stdout>) -> RenderResult {
-        let current_index = self.state.presentation().current_slide_index();
-        if self.slides_with_pending_widgets.contains(&current_index) {
-            self.render(drawer)?;
-            if self.state.presentation_mut().widgets_rendered() {
-                // Render one last time just in case it _just_ rendered
-                self.render(drawer)?;
-                self.slides_with_pending_widgets.remove(&current_index);
+    fn check_async_error(&mut self) -> bool {
+        let error_holder = self.state.presentation().state.async_error_holder();
+        let error_holder = error_holder.lock().unwrap();
+        match error_holder.deref() {
+            Some(error) => {
+                let presentation = mem::take(&mut self.state).into_presentation();
+                self.state = PresenterState::failure(
+                    &error.error,
+                    presentation,
+                    ErrorSource::Slide(error.slide),
+                    FailureMode::Other,
+                );
+                true
             }
+            None => false,
         }
-        Ok(())
+    }
+
+    fn poll_async_renders(&mut self) -> Result<bool, RenderError> {
+        if matches!(self.state, PresenterState::Failure { .. }) {
+            return Ok(false);
+        }
+        let current_index = self.state.presentation().current_slide_index();
+        if self.slides_with_pending_async_renders.contains(&current_index) {
+            let state = self.state.presentation_mut().poll_slide_async_renders();
+            match state {
+                RenderAsyncState::NotStarted | RenderAsyncState::Rendering { modified: false } => (),
+                RenderAsyncState::Rendering { modified: true } => {
+                    return Ok(true);
+                }
+                RenderAsyncState::Rendered | RenderAsyncState::JustFinishedRendering => {
+                    self.slides_with_pending_async_renders.remove(&current_index);
+                    return Ok(true);
+                }
+            };
+        }
+        Ok(false)
     }
 
     fn render(&mut self, drawer: &mut TerminalDrawer<Stdout>) -> RenderResult {
@@ -135,7 +175,7 @@ impl<'a> Presenter<'a> {
                 drawer.render_slide(presentation)?;
                 drawer.render_key_bindings(presentation)
             }
-            PresenterState::Failure { error, .. } => drawer.render_error(error),
+            PresenterState::Failure { error, source, .. } => drawer.render_error(error, source),
             PresenterState::Empty => panic!("cannot render without state"),
         };
         // If the screen is too small, simply ignore this. Eventually the user will resize the
@@ -159,8 +199,10 @@ impl<'a> Presenter<'a> {
             _ => (),
         };
         if matches!(command, Command::Redraw) {
-            let presentation = mem::take(&mut self.state).into_presentation();
-            self.state = self.validate_overflows(presentation);
+            if !self.is_displaying_other_error() {
+                let presentation = mem::take(&mut self.state).into_presentation();
+                self.state = self.validate_overflows(presentation);
+            }
             return CommandSideEffect::Redraw;
         }
 
@@ -175,14 +217,16 @@ impl<'a> Presenter<'a> {
         };
         let needs_redraw = match command {
             Command::Next => presentation.jump_next(),
+            Command::NextFast => presentation.jump_next_fast(),
             Command::Previous => presentation.jump_previous(),
+            Command::PreviousFast => presentation.jump_previous_fast(),
             Command::FirstSlide => presentation.jump_first_slide(),
             Command::LastSlide => presentation.jump_last_slide(),
             Command::GoToSlide(number) => presentation.go_to_slide(number.saturating_sub(1) as usize),
-            Command::RenderWidgets => {
-                if presentation.render_slide_widgets() {
-                    self.slides_with_pending_widgets.insert(self.state.presentation().current_slide_index());
-                    return CommandSideEffect::PollWidgets;
+            Command::RenderAsyncOperations => {
+                if presentation.trigger_slide_async_renders() {
+                    self.slides_with_pending_async_renders.insert(self.state.presentation().current_slide_index());
+                    return CommandSideEffect::Redraw;
                 } else {
                     return CommandSideEffect::None;
                 }
@@ -212,7 +256,7 @@ impl<'a> Presenter<'a> {
         if matches!(self.options.mode, PresentMode::Presentation) && !force {
             return;
         }
-        self.slides_with_pending_widgets.clear();
+        self.slides_with_pending_async_renders.clear();
         match self.load_presentation(path) {
             Ok(mut presentation) => {
                 let current = self.state.presentation();
@@ -223,24 +267,38 @@ impl<'a> Presenter<'a> {
                     presentation.go_to_slide(current.current_slide_index());
                     presentation.jump_chunk(current.current_chunk());
                 }
+                self.slides_with_pending_async_renders = match self.options.mode {
+                    PresentMode::Development | PresentMode::Presentation => {
+                        presentation.slides_with_async_renders().into_iter().collect()
+                    }
+                    // Trigger all async renders so we get snippet execution output in the PDF
+                    // file.
+                    PresentMode::Export => presentation.trigger_all_async_renders(),
+                };
                 self.state = self.validate_overflows(presentation);
             }
             Err(e) => {
                 let presentation = mem::take(&mut self.state).into_presentation();
-                self.state = PresenterState::failure(e, presentation);
+                self.state = PresenterState::failure(e, presentation, ErrorSource::Presentation, FailureMode::Other);
             }
         };
     }
 
+    fn is_displaying_other_error(&self) -> bool {
+        matches!(self.state, PresenterState::Failure { mode: FailureMode::Other, .. })
+    }
+
     fn validate_overflows(&self, presentation: Presentation) -> PresenterState {
         if self.options.validate_overflows {
             let dimensions = match WindowSize::current(self.options.font_size_fallback) {
                 Ok(dimensions) => dimensions,
-                Err(e) => return PresenterState::failure(e, presentation),
+                Err(e) => {
+                    return PresenterState::failure(e, presentation, ErrorSource::Presentation, FailureMode::Other);
+                }
             };
             match OverflowValidator::validate(&presentation, dimensions) {
                 Ok(()) => PresenterState::Presenting(presentation),
-                Err(e) => PresenterState::failure(e, presentation),
+                Err(e) => PresenterState::failure(e, presentation, ErrorSource::Presentation, FailureMode::Overflow),
             }
         } else {
             PresenterState::Presenting(presentation)
@@ -254,7 +312,8 @@ impl<'a> Presenter<'a> {
         let mut presentation = PresentationBuilder::new(
             self.default_theme,
             &mut self.resources,
-            &mut self.typst,
+            &mut self.third_party,
+            self.code_executor.clone(),
             &self.themes,
             ImageRegistry(self.image_printer.clone()),
             self.options.bindings.clone(),
@@ -294,7 +353,6 @@ impl<'a> Presenter<'a> {
 enum CommandSideEffect {
     Exit,
     Redraw,
-    PollWidgets,
     Reload,
     None,
 }
@@ -309,12 +367,19 @@ enum PresenterState {
     Failure {
         error: String,
         presentation: Presentation,
+        source: ErrorSource,
+        mode: FailureMode,
     },
 }
 
 impl PresenterState {
-    pub(crate) fn failure<E: Display>(error: E, presentation: Presentation) -> Self {
-        PresenterState::Failure { error: error.to_string(), presentation }
+    pub(crate) fn failure<E: Display>(
+        error: E,
+        presentation: Presentation,
+        source: ErrorSource,
+        mode: FailureMode,
+    ) -> Self {
+        PresenterState::Failure { error: error.to_string(), presentation, source, mode }
     }
 
     fn presentation(&self) -> &Presentation {
@@ -348,6 +413,11 @@ impl PresenterState {
     }
 }
 
+enum FailureMode {
+    Overflow,
+    Other,
+}
+
 /// This presentation mode.
 pub enum PresentMode {
     /// We are developing the presentation so we want live reloads when the input changes.
@@ -379,9 +449,6 @@ pub enum PresentationError {
     #[error(transparent)]
     Render(#[from] RenderError),
 
-    #[error(transparent)]
-    LoadPresentation(#[from] LoadPresentationError),
-
     #[error("io: {0}")]
     Io(#[from] io::Error),
 
