@@ -1,20 +1,23 @@
+use super::{code::CodeLine, execution::SnippetExecutionDisabledOperation, modals::KeyBindingsModalBuilder};
 use crate::{
     custom::{KeyBindingsConfig, OptionsConfig},
+    execute::SnippetExecutor,
     markdown::{
         elements::{
-            Code, CodeLanguage, Highlight, HighlightGroup, ListItem, ListItemType, MarkdownElement, ParagraphElement,
-            SourcePosition, Table, TableRow, Text, TextBlock,
+            Highlight, HighlightGroup, ListItem, ListItemType, MarkdownElement, ParagraphElement, Percent,
+            PercentParseError, Snippet, SnippetLanguage, SourcePosition, Table, TableRow, Text, TextBlock,
         },
         text::WeightedTextBlock,
     },
     media::{image::Image, printer::RegisterImageError, register::ImageRegistry},
     presentation::{
-        ChunkMutator, ImageProperties, MarginProperties, Modals, PreformattedLine, Presentation, PresentationMetadata,
-        PresentationState, PresentationThemeMetadata, RenderOperation, Slide, SlideBuilder, SlideChunk,
+        BlockLine, BlockLineText, ChunkMutator, ImageProperties, ImageSize, MarginProperties, Modals, Presentation,
+        PresentationMetadata, PresentationState, PresentationThemeMetadata, RenderOperation, Slide, SlideBuilder,
+        SlideChunk,
     },
     processing::{
         code::{CodePreparer, HighlightContext, HighlightMutator, HighlightedLine},
-        execution::RunCodeOperation,
+        execution::RunSnippetOperation,
         footer::{FooterContext, FooterGenerator},
         modals::IndexBuilder,
         separator::RenderSeparator,
@@ -25,18 +28,16 @@ use crate::{
     theme::{
         Alignment, AuthorPositioning, ElementType, LoadThemeError, Margin, PresentationTheme, PresentationThemeSet,
     },
-    typst::{TypstRender, TypstRenderError},
+    third_party::{ThirdPartyRender, ThirdPartyRenderError, ThirdPartyRenderRequest},
 };
 use image::DynamicImage;
 use serde::Deserialize;
 use std::{borrow::Cow, cell::RefCell, fmt::Display, iter, mem, path::PathBuf, rc::Rc, str::FromStr};
 use unicode_width::UnicodeWidthStr;
 
-use super::modals::KeyBindingsModalBuilder;
-
 // TODO: move to a theme config.
 static DEFAULT_BOTTOM_SLIDE_MARGIN: u16 = 3;
-static DEFAULT_Z_INDEX: i32 = -2;
+pub(crate) static DEFAULT_IMAGE_Z_INDEX: i32 = -2;
 
 #[derive(Default)]
 pub struct Themes {
@@ -49,11 +50,13 @@ pub struct PresentationBuilderOptions {
     pub allow_mutations: bool,
     pub implicit_slide_ends: bool,
     pub command_prefix: String,
+    pub image_attribute_prefix: String,
     pub incremental_lists: bool,
     pub force_default_theme: bool,
     pub end_slide_shorthand: bool,
     pub print_modal_background: bool,
     pub strict_front_matter_parsing: bool,
+    pub enable_snippet_execution: bool,
 }
 
 impl PresentationBuilderOptions {
@@ -66,6 +69,9 @@ impl PresentationBuilderOptions {
         if let Some(prefix) = options.command_prefix {
             self.command_prefix = prefix;
         }
+        if let Some(prefix) = options.image_attributes_prefix {
+            self.image_attribute_prefix = prefix;
+        }
     }
 }
 
@@ -75,11 +81,13 @@ impl Default for PresentationBuilderOptions {
             allow_mutations: true,
             implicit_slide_ends: false,
             command_prefix: String::default(),
+            image_attribute_prefix: "image:".to_string(),
             incremental_lists: false,
             force_default_theme: false,
             end_slide_shorthand: false,
             print_modal_background: false,
             strict_front_matter_parsing: true,
+            enable_snippet_execution: false,
         }
     }
 }
@@ -94,10 +102,12 @@ pub(crate) struct PresentationBuilder<'a> {
     chunk_mutators: Vec<Box<dyn ChunkMutator>>,
     slides: Vec<Slide>,
     highlighter: CodeHighlighter,
+    code_executor: Rc<SnippetExecutor>,
     theme: Cow<'a, PresentationTheme>,
     resources: &'a mut Resources,
-    typst: &'a mut TypstRender,
+    third_party: &'a mut ThirdPartyRender,
     slide_state: SlideState,
+    presentation_state: PresentationState,
     footer_context: Rc<RefCell<FooterContext>>,
     themes: &'a Themes,
     index_builder: IndexBuilder,
@@ -112,7 +122,8 @@ impl<'a> PresentationBuilder<'a> {
     pub(crate) fn new(
         default_theme: &'a PresentationTheme,
         resources: &'a mut Resources,
-        typst: &'a mut TypstRender,
+        third_party: &'a mut ThirdPartyRender,
+        code_executor: Rc<SnippetExecutor>,
         themes: &'a Themes,
         image_registry: ImageRegistry,
         bindings_config: KeyBindingsConfig,
@@ -124,10 +135,12 @@ impl<'a> PresentationBuilder<'a> {
             chunk_mutators: Vec::new(),
             slides: Vec::new(),
             highlighter: CodeHighlighter::default(),
+            code_executor,
             theme: Cow::Borrowed(default_theme),
             resources,
-            typst,
+            third_party,
             slide_state: Default::default(),
+            presentation_state: Default::default(),
             footer_context: Default::default(),
             themes,
             index_builder: Default::default(),
@@ -174,11 +187,10 @@ impl<'a> PresentationBuilder<'a> {
             bindings_modal_builder.set_background(background);
         };
 
-        let presentation_state = PresentationState::default();
-        let slide_index = self.index_builder.build(&self.theme, presentation_state.clone());
+        let slide_index = self.index_builder.build(&self.theme, self.presentation_state.clone());
         let bindings = bindings_modal_builder.build(&self.theme, &self.bindings_config);
         let modals = Modals { slide_index, bindings };
-        let presentation = Presentation::new(self.slides, modals, presentation_state);
+        let presentation = Presentation::new(self.slides, modals, self.presentation_state);
         Ok(presentation)
     }
 
@@ -191,11 +203,11 @@ impl<'a> PresentationBuilder<'a> {
             .as_ref()
             .or(self.theme.default_style.colors.background.as_ref())
             .and_then(Color::as_rgb);
-        // If we don't have an rgb color (or we don't have a color at all), we default to a semi
-        // transparent dark background.
+        // If we don't have an rgb color (or we don't have a color at all), we default to a dark
+        // background.
         let rgba = match color {
-            Some((r, g, b)) => [r, g, b, 230],
-            None => [0, 0, 0, 128],
+            Some((r, g, b)) => [r, g, b, 255],
+            None => [0, 0, 0, 255],
         };
         let mut image = DynamicImage::new_rgba8(1, 1);
         image.as_mut_rgba8().unwrap().get_pixel_mut(0, 0).0 = rgba;
@@ -241,12 +253,14 @@ impl<'a> PresentationBuilder<'a> {
             MarkdownElement::Heading { level, text } => self.push_heading(level, text),
             MarkdownElement::Paragraph(elements) => self.push_paragraph(elements)?,
             MarkdownElement::List(elements) => self.push_list(elements),
-            MarkdownElement::Code(code) => self.push_code(code)?,
+            MarkdownElement::Snippet(code) => self.push_code(code)?,
             MarkdownElement::Table(table) => self.push_table(table),
             MarkdownElement::ThematicBreak => self.process_thematic_break(),
             MarkdownElement::Comment { comment, source_position } => self.process_comment(comment, source_position)?,
             MarkdownElement::BlockQuote(lines) => self.push_block_quote(lines),
-            MarkdownElement::Image { path, .. } => self.push_image_from_path(path)?,
+            MarkdownElement::Image { path, title, source_position } => {
+                self.push_image_from_path(path, title, source_position)?
+            }
         };
         if should_clear_last {
             self.slide_state.last_element = LastElement::Other;
@@ -260,13 +274,20 @@ impl<'a> PresentationBuilder<'a> {
             false => serde_yaml::from_str::<PresentationMetadata>(contents),
         };
         let mut metadata = metadata.map_err(|e| BuildError::InvalidMetadata(e.to_string()))?;
+        if metadata.author.is_some() && !metadata.authors.is_empty() {
+            return Err(BuildError::InvalidMetadata("cannot have both 'author' and 'authors'".into()));
+        }
 
         if let Some(options) = metadata.options.take() {
             self.options.merge(options);
         }
         self.footer_context.borrow_mut().author = metadata.author.clone().unwrap_or_default();
         self.set_theme(&metadata.theme)?;
-        if metadata.title.is_some() || metadata.sub_title.is_some() || metadata.author.is_some() {
+        if metadata.title.is_some()
+            || metadata.sub_title.is_some()
+            || metadata.author.is_some()
+            || !metadata.authors.is_empty()
+        {
             self.push_slide_prelude();
             self.push_intro_slide(metadata);
         }
@@ -294,6 +315,9 @@ impl<'a> PresentationBuilder<'a> {
             }
         }
         if let Some(overrides) = &metadata.overrides {
+            if overrides.extends.is_some() {
+                return Err(BuildError::InvalidMetadata("theme overrides can't use 'extends'".into()));
+            }
             // This shouldn't fail as the models are already correct.
             let theme = merge_struct::merge(self.theme.as_ref(), overrides)
                 .map_err(|e| BuildError::InvalidMetadata(format!("invalid theme: {e}")))?;
@@ -321,10 +345,15 @@ impl<'a> PresentationBuilder<'a> {
             .sub_title
             .as_ref()
             .map(|text| Text::new(text.clone(), TextStyle::default().colors(styles.subtitle.colors.clone())));
-        let author = metadata
+        let authors: Vec<_> = metadata
             .author
-            .as_ref()
-            .map(|text| Text::new(text.clone(), TextStyle::default().colors(styles.author.colors.clone())));
+            .into_iter()
+            .chain(metadata.authors)
+            .map(|author| Text::new(author, TextStyle::default().colors(styles.author.colors.clone())))
+            .collect();
+        if styles.footer == Some(false) {
+            self.slide_state.ignore_footer = true;
+        }
         self.chunk_operations.push(RenderOperation::JumpToVerticalCenter);
         self.push_text(TextBlock::from(title), ElementType::PresentationTitle);
         self.push_line_break();
@@ -332,7 +361,7 @@ impl<'a> PresentationBuilder<'a> {
             self.push_text(TextBlock::from(text), ElementType::PresentationSubTitle);
             self.push_line_break();
         }
-        if let Some(text) = author {
+        if !authors.is_empty() {
             match self.theme.intro_slide.author.positioning {
                 AuthorPositioning::BelowTitle => {
                     self.push_line_break();
@@ -340,10 +369,13 @@ impl<'a> PresentationBuilder<'a> {
                     self.push_line_break();
                 }
                 AuthorPositioning::PageBottom => {
-                    self.chunk_operations.push(RenderOperation::JumpToBottomRow { index: 0 });
+                    self.chunk_operations.push(RenderOperation::JumpToBottomRow { index: authors.len() as u16 - 1 });
                 }
             };
-            self.push_text(TextBlock::from(text), ElementType::PresentationAuthor);
+            for author in authors {
+                self.push_text(TextBlock::from(author), ElementType::PresentationAuthor);
+                self.push_line_break();
+            }
         }
         self.slide_state.title = Some(TextBlock::from("[Introduction]"));
         self.terminate_slide();
@@ -396,6 +428,9 @@ impl<'a> PresentationBuilder<'a> {
             CommentCommand::IncrementalLists(value) => {
                 self.slide_state.incremental_lists = Some(value);
             }
+            CommentCommand::NoFooter => {
+                self.slide_state.ignore_footer = true;
+            }
         };
         // Don't push line breaks for any comments.
         self.slide_state.ignore_element_line_break = true;
@@ -443,13 +478,13 @@ impl<'a> PresentationBuilder<'a> {
 
         let style = self.theme.slide_title.clone();
         let mut text_style = TextStyle::default().colors(style.colors.clone());
-        if style.bold {
+        if style.bold.unwrap_or_default() {
             text_style = text_style.bold();
         }
-        if style.italics {
+        if style.italics.unwrap_or_default() {
             text_style = text_style.italics();
         }
-        if style.underlined {
+        if style.underlined.unwrap_or_default() {
             text_style = text_style.underlined();
         }
         text.apply_style(&text_style);
@@ -516,16 +551,25 @@ impl<'a> PresentationBuilder<'a> {
         }
     }
 
-    fn push_image_from_path(&mut self, path: PathBuf) -> Result<(), BuildError> {
-        let image = self.resources.image(&path)?;
-        self.push_image(image);
-        Ok(())
+    fn push_image_from_path(
+        &mut self,
+        path: PathBuf,
+        title: String,
+        source_position: SourcePosition,
+    ) -> Result<(), BuildError> {
+        let image = self.resources.image(&path).map_err(|e| BuildError::LoadImage(path, e))?;
+        self.push_image(image, title, source_position)
     }
 
-    fn push_image(&mut self, image: Image) {
+    fn push_image(&mut self, image: Image, title: String, source_position: SourcePosition) -> Result<(), BuildError> {
+        let attributes = Self::parse_image_attributes(&title, &self.options.image_attribute_prefix, source_position)?;
+        let size = match attributes.width {
+            Some(percent) => ImageSize::WidthScaled { ratio: percent.as_ratio() },
+            None => ImageSize::ShrinkIfNeeded,
+        };
         let properties = ImageProperties {
-            z_index: DEFAULT_Z_INDEX,
-            size: Default::default(),
+            z_index: DEFAULT_IMAGE_Z_INDEX,
+            size,
             restore_cursor: false,
             background_color: self.theme.default_style.colors.background,
         };
@@ -533,6 +577,7 @@ impl<'a> PresentationBuilder<'a> {
             RenderOperation::RenderImage(image, properties),
             RenderOperation::SetColors(self.theme.default_style.colors.clone()),
         ]);
+        Ok(())
     }
 
     fn push_list(&mut self, list: Vec<ListItem>) {
@@ -612,8 +657,8 @@ impl<'a> PresentationBuilder<'a> {
                 // Print a preformatted empty block so we fill in the line with properly colored
                 // spaces.
                 RenderOperation::SetColors(self.theme.block_quote.colors.base.clone()),
-                RenderOperation::RenderPreformattedLine(PreformattedLine {
-                    text: "".into(),
+                RenderOperation::RenderBlockLine(BlockLine {
+                    text: BlockLineText::Preformatted("".into()),
                     unformatted_length: 0,
                     block_length,
                     alignment: alignment.clone(),
@@ -649,11 +694,13 @@ impl<'a> PresentationBuilder<'a> {
         self.chunk_operations.push(RenderOperation::RenderLineBreak);
     }
 
-    fn push_code(&mut self, code: Code) -> Result<(), BuildError> {
+    fn push_code(&mut self, code: Snippet) -> Result<(), BuildError> {
         if code.attributes.auto_render {
             return self.push_rendered_code(code);
         }
-        let (lines, context) = self.highlight_lines(&code);
+        let lines = CodePreparer::new(&self.theme).prepare(&code);
+        let block_length = lines.iter().map(|line| line.width()).max().unwrap_or(0);
+        let (lines, context) = self.highlight_lines(&code, lines, block_length);
         for line in lines {
             self.chunk_operations.push(RenderOperation::RenderDynamic(Rc::new(line)));
         }
@@ -662,28 +709,43 @@ impl<'a> PresentationBuilder<'a> {
             self.chunk_mutators.push(Box::new(HighlightMutator::new(context)));
         }
         if code.attributes.execute {
-            self.push_code_execution(code);
+            if self.options.enable_snippet_execution {
+                self.push_code_execution(code, block_length)?;
+            } else {
+                let operation = SnippetExecutionDisabledOperation::new(
+                    self.theme.execution_output.status.failure.clone(),
+                    self.theme.code.alignment.clone().unwrap_or_default(),
+                );
+                self.chunk_operations.push(RenderOperation::RenderAsync(Rc::new(operation)))
+            }
         }
         Ok(())
     }
 
-    fn push_rendered_code(&mut self, code: Code) -> Result<(), BuildError> {
-        let image = match code.language {
-            CodeLanguage::Typst => self.typst.render_typst(&code.contents, &self.theme.typst)?,
-            CodeLanguage::Latex => self.typst.render_latex(&code.contents, &self.theme.typst)?,
-            _ => panic!("language {:?} should not be renderable", code.language),
+    fn push_rendered_code(&mut self, code: Snippet) -> Result<(), BuildError> {
+        let Snippet { contents, language, attributes } = code;
+        let error_holder = self.presentation_state.async_error_holder();
+        let request = match language {
+            SnippetLanguage::Typst => ThirdPartyRenderRequest::Typst(contents, self.theme.typst.clone()),
+            SnippetLanguage::Latex => ThirdPartyRenderRequest::Latex(contents, self.theme.typst.clone()),
+            SnippetLanguage::Mermaid => ThirdPartyRenderRequest::Mermaid(contents, self.theme.mermaid.clone()),
+            _ => panic!("language {language:?} should not be renderable"),
         };
-        self.push_image(image);
+        let operation =
+            self.third_party.render(request, &self.theme, error_holder, self.slides.len() + 1, attributes.width)?;
+        self.chunk_operations.push(operation);
         Ok(())
     }
 
-    fn highlight_lines(&self, code: &Code) -> (Vec<HighlightedLine>, Rc<RefCell<HighlightContext>>) {
-        let lines = CodePreparer::new(&self.theme).prepare(code);
-        let block_length = lines.iter().map(|line| line.width()).max().unwrap_or(0);
-        let mut empty_highlighter = self.highlighter.language_highlighter(&CodeLanguage::Unknown);
+    fn highlight_lines(
+        &self,
+        code: &Snippet,
+        lines: Vec<CodeLine>,
+        block_length: usize,
+    ) -> (Vec<HighlightedLine>, Rc<RefCell<HighlightContext>>) {
         let mut code_highlighter = self.highlighter.language_highlighter(&code.language);
-        let padding_style = {
-            let mut highlighter = self.highlighter.language_highlighter(&CodeLanguage::Rust);
+        let dim_style = {
+            let mut highlighter = self.highlighter.language_highlighter(&SnippetLanguage::Rust);
             highlighter.style_line("//").next().expect("no styles").style
         };
         let groups = match self.options.allow_mutations {
@@ -700,8 +762,8 @@ impl<'a> PresentationBuilder<'a> {
         let mut output = Vec::new();
         let block_style = &self.theme.code;
         for line in lines.into_iter() {
-            let highlighted = line.highlight(&padding_style, &mut code_highlighter, block_style);
-            let not_highlighted = line.highlight(&padding_style, &mut empty_highlighter, block_style);
+            let highlighted = line.highlight(&dim_style, &mut code_highlighter, block_style);
+            let not_highlighted = line.dim(&dim_style, block_style);
             let width = line.width();
             let line_number = line.line_number;
             let context = context.clone();
@@ -710,14 +772,14 @@ impl<'a> PresentationBuilder<'a> {
         (output, context)
     }
 
-    fn push_code_execution(&mut self, code: Code) {
-        let operation = RunCodeOperation::new(
-            code,
-            self.theme.default_style.colors.clone(),
-            self.theme.execution_output.colors.clone(),
-        );
-        let operation = RenderOperation::RenderOnDemand(Rc::new(operation));
+    fn push_code_execution(&mut self, code: Snippet, block_length: usize) -> Result<(), BuildError> {
+        if !self.code_executor.is_execution_supported(&code.language) {
+            return Err(BuildError::UnsupportedExecution(code.language));
+        }
+        let operation = RunSnippetOperation::new(code, self.code_executor.clone(), &self.theme, block_length as u16);
+        let operation = RenderOperation::RenderAsync(Rc::new(operation));
         self.chunk_operations.push(operation);
+        Ok(())
     }
 
     fn terminate_slide(&mut self) {
@@ -738,6 +800,9 @@ impl<'a> PresentationBuilder<'a> {
     }
 
     fn generate_footer(&mut self) -> Vec<RenderOperation> {
+        if self.slide_state.ignore_footer {
+            return Vec::new();
+        }
         let generator = FooterGenerator {
             style: self.theme.footer.clone().unwrap_or_default(),
             current_slide: self.slides.len(),
@@ -802,11 +867,43 @@ impl<'a> PresentationBuilder<'a> {
         }
         flattened_row
     }
+
+    fn parse_image_attributes(
+        input: &str,
+        attribute_prefix: &str,
+        source_position: SourcePosition,
+    ) -> Result<ImageAttributes, BuildError> {
+        let mut attributes = ImageAttributes::default();
+        for attribute in input.split(',') {
+            let Some((prefix, suffix)) = attribute.split_once(attribute_prefix) else { continue };
+            if !prefix.is_empty() || (attribute_prefix.is_empty() && suffix.is_empty()) {
+                continue;
+            }
+            Self::parse_image_attribute(suffix, &mut attributes)
+                .map_err(|e| BuildError::ImageAttributeParse { line: source_position.start.line + 1, error: e })?;
+        }
+        Ok(attributes)
+    }
+
+    fn parse_image_attribute(input: &str, attributes: &mut ImageAttributes) -> Result<(), ImageAttributeError> {
+        let Some((key, value)) = input.split_once(':') else {
+            return Err(ImageAttributeError::AttributeMissing);
+        };
+        match key {
+            "width" | "w" => {
+                let width = value.parse().map_err(ImageAttributeError::InvalidWidth)?;
+                attributes.width = Some(width);
+                Ok(())
+            }
+            _ => Err(ImageAttributeError::UnknownAttribute(key.to_string())),
+        }
+    }
 }
 
 #[derive(Debug, Default)]
 struct SlideState {
     ignore_element_line_break: bool,
+    ignore_footer: bool,
     needs_enter_column: bool,
     last_chunk_ended_in_list: bool,
     last_element: LastElement,
@@ -841,10 +938,10 @@ enum LastElement {
 /// An error when building a presentation.
 #[derive(thiserror::Error, Debug)]
 pub enum BuildError {
-    #[error("loading image: {0}")]
-    LoadImage(#[from] LoadImageError),
+    #[error("failed to load image '{0}': {1}")]
+    LoadImage(PathBuf, LoadImageError),
 
-    #[error("registering image: {0}")]
+    #[error("failed to register image: {0}")]
     RegisterImage(#[from] RegisterImageError),
 
     #[error("invalid presentation metadata: {0}")]
@@ -874,8 +971,14 @@ pub enum BuildError {
     #[error("error parsing command at line {line}: {error}")]
     CommandParse { line: usize, error: CommandParseError },
 
-    #[error("typst render failed: {0}")]
-    TypstRender(#[from] TypstRenderError),
+    #[error("error parsing image attribute at line {line}: {error}")]
+    ImageAttributeParse { line: usize, error: ImageAttributeError },
+
+    #[error("third party render failed: {0}")]
+    ThirdPartyRender(#[from] ThirdPartyRenderError),
+
+    #[error("language {0:?} does not support execution")]
+    UnsupportedExecution(SnippetLanguage),
 }
 
 #[derive(Debug, Clone, PartialEq, Deserialize)]
@@ -893,6 +996,7 @@ enum CommentCommand {
     ResetLayout,
     JumpToMiddle,
     IncrementalLists(bool),
+    NoFooter,
 }
 
 impl FromStr for CommentCommand {
@@ -983,6 +1087,9 @@ struct StrictPresentationMetadata {
     #[serde(default)]
     author: Option<String>,
 
+    #[serde(default)]
+    authors: Vec<String>,
+
     #[serde(default)]
     theme: PresentationThemeMetadata,
 
@@ -992,14 +1099,32 @@ struct StrictPresentationMetadata {
 
 impl From<StrictPresentationMetadata> for PresentationMetadata {
     fn from(strict: StrictPresentationMetadata) -> Self {
-        let StrictPresentationMetadata { title, sub_title, author, theme, options } = strict;
-        Self { title, sub_title, author, theme, options }
+        let StrictPresentationMetadata { title, sub_title, author, authors, theme, options } = strict;
+        Self { title, sub_title, author, authors, theme, options }
     }
 }
 
+#[derive(thiserror::Error, Debug)]
+pub enum ImageAttributeError {
+    #[error("invalid width: {0}")]
+    InvalidWidth(PercentParseError),
+
+    #[error("no attribute given")]
+    AttributeMissing,
+
+    #[error("unknown attribute: '{0}'")]
+    UnknownAttribute(String),
+}
+
+#[derive(Clone, Debug, Default, PartialEq)]
+struct ImageAttributes {
+    width: Option<Percent>,
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
+    use crate::markdown::elements::SnippetAttributes;
     use rstest::rstest;
 
     fn build_presentation(elements: Vec<MarkdownElement>) -> Presentation {
@@ -1023,13 +1148,15 @@ mod test {
     ) -> Result<Presentation, BuildError> {
         let theme = PresentationTheme::default();
         let mut resources = Resources::new("/tmp", Default::default());
-        let mut typst = TypstRender::default();
+        let mut third_party = ThirdPartyRender::default();
+        let code_executor = Rc::new(SnippetExecutor::default());
         let themes = Themes::default();
         let bindings = KeyBindingsConfig::default();
         let builder = PresentationBuilder::new(
             &theme,
             &mut resources,
-            &mut typst,
+            &mut third_party,
+            code_executor,
             &themes,
             Default::default(),
             bindings,
@@ -1070,9 +1197,9 @@ mod test {
             RenderText { .. }
             | RenderLineBreak
             | RenderImage(_, _)
-            | RenderPreformattedLine(_)
+            | RenderBlockLine(_)
             | RenderDynamic(_)
-            | RenderOnDemand(_) => true,
+            | RenderAsync(_) => true,
         }
     }
 
@@ -1098,7 +1225,7 @@ mod test {
     }
 
     fn extract_slide_text_lines(slide: Slide) -> Vec<String> {
-        let operations: Vec<_> = slide.into_operations().into_iter().filter(|op| is_visible(op)).collect();
+        let operations: Vec<_> = slide.into_operations().into_iter().filter(is_visible).collect();
         extract_text_lines(&operations)
     }
 
@@ -1111,11 +1238,11 @@ mod test {
             MarkdownElement::Heading { text: TextBlock::from("bye"), level: 1 },
         ];
         let presentation = build_presentation(elements);
-        for (index, slide) in presentation.iter_slides().into_iter().enumerate() {
+        for (index, slide) in presentation.iter_slides().enumerate() {
             let clear_screen_count =
-                slide.iter_operations().filter(|op| matches!(op, RenderOperation::ClearScreen)).count();
+                slide.iter_visible_operations().filter(|op| matches!(op, RenderOperation::ClearScreen)).count();
             let set_colors_count =
-                slide.iter_operations().filter(|op| matches!(op, RenderOperation::SetColors(_))).count();
+                slide.iter_visible_operations().filter(|op| matches!(op, RenderOperation::SetColors(_))).count();
             assert_eq!(clear_screen_count, 1, "{clear_screen_count} clear screens in slide {index}");
             assert_eq!(set_colors_count, 1, "{set_colors_count} clear screens in slide {index}");
         }
@@ -1328,7 +1455,7 @@ mod test {
         ];
         let slides = build_presentation(elements).into_slides();
         let first_chunk = &slides[0];
-        let operations = first_chunk.iter_operations().collect::<Vec<_>>();
+        let operations = first_chunk.iter_visible_operations().collect::<Vec<_>>();
         // This is pretty easy to break, refactor soon
         let last_operation = &operations[operations.len() - 4];
         assert!(matches!(last_operation, RenderOperation::RenderLineBreak), "last operation is {last_operation:?}");
@@ -1402,8 +1529,9 @@ mod test {
         let presentation = build_presentation_with_options(elements, options);
         assert_eq!(presentation.iter_slides().count(), 2);
 
-        let second = presentation.iter_slides().skip(1).next().unwrap();
-        let before_text = second.iter_operations().take_while(|e| !matches!(e, RenderOperation::RenderText { .. }));
+        let second = presentation.iter_slides().nth(1).unwrap();
+        let before_text =
+            second.iter_visible_operations().take_while(|e| !matches!(e, RenderOperation::RenderText { .. }));
         let break_count = before_text.filter(|e| matches!(e, RenderOperation::RenderLineBreak)).count();
         assert_eq!(break_count, 1);
     }
@@ -1415,4 +1543,65 @@ mod test {
         let result = try_build_presentation_with_options(elements, options);
         assert!(result.is_ok());
     }
+
+    #[rstest]
+    #[case::enabled(true)]
+    #[case::disabled(false)]
+    fn snippet_execution(#[case] enabled: bool) {
+        let element = MarkdownElement::Snippet(Snippet {
+            contents: "".into(),
+            language: SnippetLanguage::Rust,
+            attributes: SnippetAttributes { execute: true, ..Default::default() },
+        });
+        let options = PresentationBuilderOptions { enable_snippet_execution: enabled, ..Default::default() };
+        let presentation = build_presentation_with_options(vec![element], options);
+        let slide = presentation.iter_slides().next().unwrap();
+        let mut found_render_block = false;
+        let mut found_cant_render_block = false;
+        for operation in slide.iter_visible_operations() {
+            match operation {
+                RenderOperation::RenderAsync(operation) => {
+                    let operation = format!("{operation:?}");
+                    if operation.contains("RunSnippetOperation") {
+                        assert!(enabled);
+                        found_render_block = true;
+                    } else if operation.contains("SnippetExecutionDisabledOperation") {
+                        assert!(!enabled);
+                        found_cant_render_block = true;
+                    }
+                }
+                _ => (),
+            };
+        }
+        if found_render_block {
+            assert!(enabled, "snippet execution block found but not enabled");
+        } else {
+            assert!(!enabled, "snippet execution enabled but not found");
+        }
+        if found_cant_render_block {
+            assert!(!enabled, "can't execute snippet operation found but enabled");
+        } else {
+            assert!(enabled, "can't execute snippet operation not found");
+        }
+    }
+
+    #[rstest]
+    #[case::width("image:width:50%", Some(50))]
+    #[case::w("image:w:50%", Some(50))]
+    #[case::nothing("", None)]
+    #[case::no_prefix("width", None)]
+    fn image_attributes(#[case] input: &str, #[case] expectation: Option<u8>) {
+        let attributes =
+            PresentationBuilder::parse_image_attributes(&input, "image:", Default::default()).expect("failed to parse");
+        assert_eq!(attributes.width, expectation.map(Percent));
+    }
+
+    #[rstest]
+    #[case::width("width:50%", Some(50))]
+    #[case::empty("", None)]
+    fn image_attributes_empty_prefix(#[case] input: &str, #[case] expectation: Option<u8>) {
+        let attributes =
+            PresentationBuilder::parse_image_attributes(input, "", Default::default()).expect("failed to parse");
+        assert_eq!(attributes.width, expectation.map(Percent));
+    }
 }
