@@ -1,7 +1,7 @@
 use super::padding::NumberPadder;
 use crate::{
-    markdown::elements::{Code, HighlightGroup},
-    presentation::{AsRenderOperations, ChunkMutator, PreformattedLine, RenderOperation},
+    markdown::elements::{HighlightGroup, Snippet},
+    presentation::{AsRenderOperations, BlockLine, BlockLineText, ChunkMutator, RenderOperation},
     render::{
         highlighting::{LanguageHighlighter, StyledTokens},
         properties::WindowSize,
@@ -22,7 +22,7 @@ impl<'a> CodePreparer<'a> {
         Self { theme }
     }
 
-    pub(crate) fn prepare(&self, code: &Code) -> Vec<CodeLine> {
+    pub(crate) fn prepare(&self, code: &Snippet) -> Vec<CodeLine> {
         let mut lines = Vec::new();
         let horizontal_padding = self.theme.code.padding.horizontal.unwrap_or(0);
         let vertical_padding = self.theme.code.padding.vertical.unwrap_or(0);
@@ -36,14 +36,14 @@ impl<'a> CodePreparer<'a> {
         lines
     }
 
-    fn push_lines(&self, code: &Code, horizontal_padding: u8, lines: &mut Vec<CodeLine>) {
+    fn push_lines(&self, code: &Snippet, horizontal_padding: u8, lines: &mut Vec<CodeLine>) {
         if code.contents.is_empty() {
             return;
         }
 
         let padding = " ".repeat(horizontal_padding as usize);
-        let padder = NumberPadder::new(code.contents.lines().count());
-        for (index, line) in code.contents.lines().enumerate() {
+        let padder = NumberPadder::new(code.visible_lines().count());
+        for (index, line) in code.visible_lines().enumerate() {
             let mut line = line.to_string();
             let mut prefix = padding.clone();
             if code.attributes.line_numbers {
@@ -76,13 +76,21 @@ impl CodeLine {
 
     pub(crate) fn highlight(
         &self,
-        padding_style: &Style,
+        dim_style: &Style,
         code_highlighter: &mut LanguageHighlighter,
         block_style: &CodeBlockStyle,
     ) -> String {
-        let mut output = StyledTokens { style: *padding_style, tokens: &self.prefix }.apply_style(block_style);
+        let mut output = StyledTokens { style: *dim_style, tokens: &self.prefix }.apply_style(block_style);
         output.push_str(&code_highlighter.highlight_line(&self.code, block_style));
-        output.push_str(&StyledTokens { style: *padding_style, tokens: &self.suffix }.apply_style(block_style));
+        output.push_str(&StyledTokens { style: *dim_style, tokens: &self.suffix }.apply_style(block_style));
+        output
+    }
+
+    pub(crate) fn dim(&self, padding_style: &Style, block_style: &CodeBlockStyle) -> String {
+        let mut output = String::new();
+        for chunk in [&self.prefix, &self.code, &self.suffix] {
+            output.push_str(&StyledTokens { style: *padding_style, tokens: chunk }.apply_style(block_style));
+        }
         output
     }
 }
@@ -115,8 +123,8 @@ impl AsRenderOperations for HighlightedLine {
             false => self.not_highlighted.clone(),
         };
         vec![
-            RenderOperation::RenderPreformattedLine(PreformattedLine {
-                text,
+            RenderOperation::RenderBlockLine(BlockLine {
+                text: BlockLineText::Preformatted(text),
                 unformatted_length: self.width as u16,
                 block_length: context.block_length as u16,
                 alignment: context.alignment.clone(),
@@ -179,7 +187,7 @@ impl ChunkMutator for HighlightMutator {
 
 #[cfg(test)]
 mod test {
-    use crate::markdown::elements::{CodeAttributes, CodeLanguage};
+    use crate::markdown::elements::{SnippetAttributes, SnippetLanguage};
 
     use super::*;
 
@@ -187,10 +195,10 @@ mod test {
     fn code_with_line_numbers() {
         let total_lines = 11;
         let input_lines = "hi\n".repeat(total_lines);
-        let code = Code {
+        let code = Snippet {
             contents: input_lines,
-            language: CodeLanguage::Unknown,
-            attributes: CodeAttributes { line_numbers: true, ..Default::default() },
+            language: SnippetLanguage::Unknown("".to_string()),
+            attributes: SnippetAttributes { line_numbers: true, ..Default::default() },
         };
         let lines = CodePreparer { theme: &Default::default() }.prepare(&code);
         assert_eq!(lines.len(), total_lines);
