@@ -117,7 +117,9 @@ impl KeyBindingsModalBuilder {
         let mut builder = ModalBuilder::new("Key bindings");
         builder.content.extend([
             Self::build_line("Next", &config.next),
+            Self::build_line("Next (fast)", &config.next_fast),
             Self::build_line("Previous", &config.previous),
+            Self::build_line("Previous (fast)", &config.previous_fast),
             Self::build_line("First slide", &config.first_slide),
             Self::build_line("Last slide", &config.last_slide),
             Self::build_line("Go to slide", &config.go_to_slide),
